@@ -2,9 +2,8 @@ ${header}
 namespace Boo.Lang.Compiler.Ast
 {
 	using System;
-	
-	[Serializable]
-	public class ${node.Name} : Boo.Lang.Compiler.Ast.Impl.${node.Name}Impl
+
+	public partial class ${node.Name}
 	{
 		public ${node.Name}()
 		{
