@@ -30,17 +30,16 @@ using System.Reflection;
 using System.Security.Permissions;
 
 [assembly: ReflectionPermission(SecurityAction.RequestMinimum,
-								ReflectionEmit=true,
-								TypeInformation=true)]
+								ReflectionEmit=true)]
 [assembly: AssemblyTitle("boo - an extensible programming language for the CLI")]
 [assembly: AssemblyDescription("")]
 [assembly: AssemblyConfiguration("")]
 [assembly: AssemblyCompany("")]
 [assembly: AssemblyProduct("boo - an extensible programming language for the CLI")]
-[assembly: AssemblyCopyright("(C) 2003-2005 Rodrigo Barreto de Oliveira")]
+[assembly: AssemblyCopyright("(C) 2003-2007 Rodrigo Barreto de Oliveira")]
 [assembly: AssemblyTrademark("")]
 [assembly: AssemblyCulture("")]
-[assembly: AssemblyVersion("1.0.0.0")]
+[assembly: AssemblyVersion("2.0.0.0")]
 [assembly: AssemblyDelaySign(false)]
 #if !IGNOREKEYFILE
 [assembly: AssemblyKeyFile("../src/boo.snk")]
