@@ -35,7 +35,6 @@ using Boo.Lang.Compiler.Ast;
 namespace Boo.Lang.Compiler.Ast.Visitors
 {
 	/// <summary>
-	/// Imprime uma AST boo em boo.
 	/// </summary>
 	public class BooPrinterVisitor : TextEmitter
 	{
@@ -43,12 +42,13 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 		public enum PrintOptions
 		{
 			None,
-			PrintLocals = 1
+			PrintLocals = 1,
+			WSA = 2,
 		}
 
 		static Regex _identifierRE = new Regex("^[a-zA-Z.]+$");
 		
-		static Regex _extendedRE = new Regex(@"\s");
+		//static Regex _extendedRE = new Regex(@"\s");
 
 		public PrintOptions Options = PrintOptions.None;
 		
@@ -125,12 +125,13 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 			WriteLine();
 		}
 		
-		bool IsExtendedRE(string s)
+		static bool IsExtendedRE(string s)
 		{
-			return _extendedRE.IsMatch(s);
+			return s.IndexOfAny(new char[] { ' ', '\t' }) > -1;
+			//return _extendedRE.IsMatch(s);
 		}
 		
-		bool IsSimpleIdentifier(string s)
+		static bool IsSimpleIdentifier(string s)
 		{
 			return _identifierRE.IsMatch(s);
 		}
@@ -160,21 +161,60 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 			}
 			WriteLine();
 		}
-
-		public void WriteBlock(Block b)
+		
+		public bool IsWhiteSpaceAgnostic
 		{
-			Indent();
-			if (0 == b.Statements.Count)
+			get { return IsOptionSet(PrintOptions.WSA); }
+		}
+		
+		private void WritePass()
+		{
+			if (!IsWhiteSpaceAgnostic)
 			{
 				WriteIndented();
 				WriteKeyword("pass");
 				WriteLine();
 			}
+		}
+		
+		private void WriteBlockStatements(Block b)
+		{
+			if (0 == b.Statements.Count)
+			{
+				WritePass();
+			}
 			else
 			{
 				Visit(b.Statements);
 			}
+		}
+
+		public void WriteBlock(Block b)
+		{
+			BeginBlock();
+			WriteBlockStatements(b);
+			EndBlock();
+		}
+		
+		private void BeginBlock()
+		{
+			Indent();
+		}
+		
+		private void EndBlock()
+		{
 			Dedent();
+			if (IsWhiteSpaceAgnostic)
+			{
+				WriteEnd();
+			}
+		}
+		
+		private void WriteEnd()
+		{
+			WriteIndented();
+			WriteKeyword("end");
+			WriteLine();
 		}
 		
 		override public void OnAttribute(Attribute att)
@@ -205,13 +245,18 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 		override public void OnEvent(Event node)
 		{
 			WriteAttributes(node.Attributes, true);
-			WriteModifiers(node);
+			WriteOptionalModifiers(node);
 			WriteKeyword("event ");
 			Write(node.Name);
 			WriteTypeReference(node.Type);
 			WriteLine();
 		}
 
+		private static bool IsInterfaceMember(TypeMember node)
+		{
+			return node.DeclaringType != null && node.DeclaringType.NodeType == NodeType.InterfaceDefinition;
+		}
+
 		override public void OnField(Field f)
 		{
 			WriteAttributes(f.Attributes, true);
@@ -234,8 +279,10 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 		
 		override public void OnProperty(Property node)
 		{
+			bool interfaceMember = IsInterfaceMember(node);
+
 			WriteAttributes(node.Attributes, true);
-			WriteModifiers(node);
+			WriteOptionalModifiers(node);
 			WriteIndented("");
 			Visit(node.ExplicitInfo);
 			Write(node.Name);
@@ -245,26 +292,37 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 			}
 			WriteTypeReference(node.Type);
 			WriteLine(":");
-			Indent();
-			if (null != node.Getter)
-			{
-				WriteAttributes(node.Getter.Attributes, true);
-				WriteModifiers(node.Getter);
-				WriteKeyword("get");
-				WriteLine(":");
-				WriteBlock(node.Getter.Body);
-			}
-			if (null != node.Setter)
-			{
-				WriteAttributes(node.Setter.Attributes, true);
-				WriteModifiers(node.Setter);
-				WriteKeyword("set");
-				WriteLine(":");
-				WriteBlock(node.Setter.Body);
-			}
-			Dedent();
+			BeginBlock();
+			WritePropertyAccessor(node.Getter, "get", interfaceMember);
+			WritePropertyAccessor(node.Setter, "set", interfaceMember);
+			EndBlock();
 		}
-		
+
+		private void WritePropertyAccessor(Method method, string name, bool interfaceMember)
+		{
+			if (null == method) return;
+			WriteAttributes(method.Attributes, true);
+			if (interfaceMember)
+			{
+				WriteIndented();
+			}
+			else
+			{
+				WriteModifiers(method);
+			}
+			WriteKeyword(name);
+			if (interfaceMember)
+			{
+				WriteLine();
+			}
+			else
+			{
+				WriteLine(":");
+				WriteBlock(method.Body);
+			}
+			
+		}
+
 		override public void OnEnumMember(EnumMember node)
 		{
 			WriteAttributes(node.Attributes, true);
@@ -286,8 +344,8 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 		{
 			OnMethod(c);
 		}
-		
-		bool IsSimpleClosure(CallableBlockExpression node)
+
+		bool IsSimpleClosure(BlockExpression node)
 		{
 			if (1 == node.Body.Statements.Count)
 			{
@@ -312,8 +370,8 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 			}
 			return false;
 		}
-		
-		override public void OnCallableBlockExpression(CallableBlockExpression node)
+
+		override public void OnBlockExpression(BlockExpression node)
 		{
 			if (IsSimpleClosure(node))
 			{
@@ -341,7 +399,8 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 		void WriteCallableDefinitionHeader(string keyword, CallableDefinition node)
 		{
 			WriteAttributes(node.Attributes, true);
-			WriteModifiers(node);
+			WriteOptionalModifiers(node);
+
 			WriteKeyword(keyword);
 
 			IExplicitMember em = node as IExplicitMember;
@@ -350,6 +409,10 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 				Visit(em.ExplicitInfo);
 			}
 			Write(node.Name);
+			if (node.GenericParameters.Count > 0)
+			{
+				WriteGenericParameterList(node.GenericParameters);
+			}
 			WriteParameterList(node.Parameters);
 			WriteTypeReference(node.ReturnType);
 			if (node.ReturnTypeAttributes.Count > 0)
@@ -358,7 +421,19 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 				WriteAttributes(node.ReturnTypeAttributes, false);
 			}
 		}
-		
+
+		private void WriteOptionalModifiers(TypeMember node)
+		{
+			if (IsInterfaceMember(node))
+			{
+				WriteIndented();
+			}
+			else
+			{
+				WriteModifiers(node);
+			}
+		}
+
 		override public void OnCallableDefinition(CallableDefinition node)
 		{
 			WriteCallableDefinitionHeader("callable ", node);
@@ -366,18 +441,19 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 
 		override public void OnMethod(Method m)
 		{
-            if (m.IsRuntime)
-            {
-                WriteImplementationComment("runtime");
-            }
-			if (m.IsExtension)
-			{
-				WriteImplementationComment("extension");
-			}
+			if (m.IsRuntime) WriteImplementationComment("runtime");
+
 			WriteCallableDefinitionHeader("def ", m);
-			WriteLine(":");
-			WriteLocals(m);
-			WriteBlock(m.Body);
+			if (IsInterfaceMember(m))
+			{
+				WriteLine();
+			}
+			else
+			{
+				WriteLine(":");
+				WriteLocals(m);
+				WriteBlock(m.Body);
+			}
 		}
 		
 		private void WriteImplementationComment(string comment)
@@ -426,6 +502,11 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 				WriteTypeReference(p.Type);
 			}
 		}
+
+		override public void OnGenericParameterDeclaration(GenericParameterDeclaration gp)
+		{
+			Write(gp.Name);
+		}
 		
 		override public void OnTypeofExpression(TypeofExpression node)
 		{
@@ -438,13 +519,24 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 		{
 			Write(t.Name);
 		}
-		
+			
 		override public void OnGenericTypeReference(GenericTypeReference node)
 		{
 			OnSimpleTypeReference(node);
 			WriteGenericArguments(node.GenericArguments);
 		}
 		
+		override public void OnGenericTypeDefinitionReference(GenericTypeDefinitionReference node)
+		{
+			OnSimpleTypeReference(node);
+			Write("[of *");
+			for (int i = 1; i < node.GenericPlaceholders; i++)
+			{
+				Write(", *");
+			}
+			Write("]");						
+		}
+		
 		override public void OnGenericReferenceExpression(GenericReferenceExpression node)
 		{
 			Visit(node.Target);
@@ -602,7 +694,15 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 			Write(" ");
 			WriteOperator(GetBinaryOperatorText(e.Operator));
 			Write(" ");
-			Visit(e.Right);
+			if (e.Operator == BinaryOperatorType.TypeTest)
+			{
+				// isa rhs is encoded in a typeof expression
+				Visit(((TypeofExpression)e.Right).Type);
+			}
+			else
+			{
+				Visit(e.Right);
+			}
 			if (needsParens)
 			{
 				Write(")");
@@ -726,22 +826,44 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 			Write(e.Value);
 		}
 		
-		override public void OnAstLiteralExpression(AstLiteralExpression e)
+		override public void OnSpliceExpression(SpliceExpression e)
+		{
+			WriteOperator("$(");
+			Visit(e.Expression);
+			WriteOperator(")");
+		}
+		
+		override public void OnSpliceTypeReference(SpliceTypeReference node)
+		{
+			WriteOperator("$(");
+			Visit(node.Expression);
+			WriteOperator(")");
+		}
+		
+		void WriteIndentedOperator(string op)
 		{
 			WriteIndented();
-			WriteKeyword("ast");
+			WriteOperator(op);
+		}
+		
+		override public void OnQuasiquoteExpression(QuasiquoteExpression e)
+		{
+			WriteIndentedOperator("[|");
 			if (e.Node is Expression)
 			{
-				Write(" { ");
+				Write(" ");
 				Visit(e.Node);
-				Write(" }");
+				Write(" ");
+				WriteIndentedOperator("|]");
 			}
 			else
 			{
-				WriteLine(":");
+				WriteLine();
 				Indent();
 				Visit(e.Node);
 				Dedent();
+				WriteIndentedOperator("|]");
+				WriteLine();
 			}
 		}
 
@@ -864,13 +986,6 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 			WriteBlock(fs.Block);
 		}
 		
-		override public void OnRetryStatement(RetryStatement node)
-		{
-			WriteIndented();
-			WriteKeyword("retry");
-			WriteLine();
-		}
-		
 		override public void OnTryStatement(TryStatement node)
 		{
 			WriteIndented();
@@ -878,13 +993,6 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 			WriteLine();
 			WriteBlock(node.ProtectedBlock);
 			Visit(node.ExceptionHandlers);
-			if (null != node.SuccessBlock)
-			{
-				WriteIndented();
-				WriteKeyword("success:");
-				WriteLine();
-				WriteBlock(node.SuccessBlock);
-			}
 			if (null != node.EnsureBlock)
 			{
 				WriteIndented();
@@ -942,22 +1050,56 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 			WriteConditionalBlock("while", node.Condition, node.Block);
 		}
 
-		override public void OnIfStatement(IfStatement ifs)
-		{
-			WriteIndented();
-			WriteKeyword("if ");
-			Visit(ifs.Condition);
-			WriteLine(":");
-			WriteBlock(ifs.TrueBlock);
-			if (null != ifs.FalseBlock)
+		override public void OnIfStatement(IfStatement node)
+		{	
+			WriteIfBlock("if ", node);
+			Block elseBlock = WriteElifs(node);
+			if (null != elseBlock)
 			{
 				WriteIndented();
 				WriteKeyword("else:");
 				WriteLine();
-				WriteBlock(ifs.FalseBlock);
+				WriteBlock(elseBlock);
+			}
+			else
+			{
+				if (IsWhiteSpaceAgnostic)
+				{
+					WriteEnd();
+				}
 			}
 		}
-		
+
+		private Block WriteElifs(IfStatement node)
+		{
+			Block falseBlock = node.FalseBlock;
+			while (IsElif(falseBlock))
+			{
+				IfStatement stmt = (IfStatement) falseBlock.Statements[0];
+				WriteIfBlock("elif ", stmt);
+				falseBlock = stmt.FalseBlock;
+			}
+			return falseBlock;
+		}
+
+		private void WriteIfBlock(string keyword, IfStatement ifs)
+		{
+			WriteIndented();
+			WriteKeyword(keyword);
+			Visit(ifs.Condition);
+			WriteLine(":");
+			Indent();
+			WriteBlockStatements(ifs.TrueBlock);
+			Dedent();
+		}
+
+		private static bool IsElif(Block block)
+		{
+			if (block == null) return false;
+			if (block.Statements.Count != 1) return false;
+			return block.Statements[0] is IfStatement;
+		}
+
 		override public void OnDeclarationStatement(DeclarationStatement d)
 		{
 			WriteIndented();
@@ -1385,7 +1527,8 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 		{
 			Write(st);
 			int last = items.Count-1;
-			for (int i=0; i<items.Count; ++i)
+			int i = 0;
+			foreach (ParameterDeclaration item in items)
 			{
 				if (i > 0)
 				{
@@ -1395,11 +1538,19 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 				{
 					Write("*");
 				}
-				Visit(items.GetNodeAt(i));
+				Visit(item);
+				++i;
 			}
 			Write(ed);
 		}
 		
+		void WriteGenericParameterList(GenericParameterDeclarationCollection items)
+		{
+			Write("[of ");
+			WriteCommaSeparatedList(items);
+			Write("]");
+		}
+		
 		void WriteAttribute(Attribute attribute)
 		{
 			WriteAttribute(attribute, null);
@@ -1496,7 +1647,7 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 			}
 		}
 
-		void WriteTypeDefinition(string keyword, TypeDefinition td)
+		virtual protected void WriteTypeDefinition(string keyword, TypeDefinition td)
 		{
 			WriteAttributes(td.Attributes, true);
 			WriteModifiers(td);
@@ -1504,18 +1655,21 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 			WriteKeyword(keyword);
 			Write(" ");
 			Write(td.Name);
+
+			if (td.GenericParameters.Count != 0)
+			{
+				WriteGenericParameterList(td.GenericParameters);
+			}
+
 			if (td.BaseTypes.Count > 0)
 			{
 				Write("(");
-				for (int i=0; i<td.BaseTypes.Count; ++i)
-				{
-					if (i > 0) { Write(", "); }
-					Write(((SimpleTypeReference)td.BaseTypes[i]).Name);
-				}
+				WriteCommaSeparatedList<TypeReference>(td.BaseTypes);
 				Write(")");
 			}
+
 			WriteLine(":");
-			Indent();
+			BeginBlock();
 			if (td.Members.Count > 0)
 			{
 				foreach (TypeMember member in td.Members)
@@ -1526,11 +1680,9 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 			}
 			else
 			{
-				WriteIndented();
-				WriteKeyword("pass");
-				WriteLine();
+				WritePass();
 			}
-			Dedent();
+			EndBlock();
 		}
 		
 		bool WasOmitted(Expression node)
