@@ -1,4 +1,5 @@
 #region license
+
 // Copyright (c) 2004, Rodrigo B. de Oliveira (rbo@acm.org)
 // All rights reserved.
 // 
@@ -24,95 +25,96 @@
 // CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 // OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 // THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
 #endregion
 
 using System;
-using System.Diagnostics;
 using System.Collections;
+using System.Diagnostics;
 using System.IO;
 using System.Reflection;
-using System.Globalization;
-using Boo.Lang;
+using System.Text.RegularExpressions;
 
 namespace Boo.Lang.Compiler
 {
 	/// <summary>
 	/// Compiler parameters.
 	/// </summary>
-	public class CompilerParameters : System.MarshalByRefObject
+	public class CompilerParameters
 	{
-		static private bool _NET_2_0 = Environment.Version >= new Version(2, 0);
-		
-		static private bool _NET_1_1 = (Environment.Version.Major == 1 &&
-						Environment.Version.Minor == 1);
-		
-		TextWriter _outputWriter;
-		
-		CompilerPipeline _pipeline;
+		private static List _validFileExtensions = new List(new string[] {".dll", ".exe"});
 
-		CompilerInputCollection _input;
-		
-		CompilerResourceCollection _resources;
+		private TextWriter _outputWriter;
 
-		AssemblyCollection _assemblyReferences;
+		private CompilerPipeline _pipeline;
+
+		private CompilerInputCollection _input;
+
+		private CompilerResourceCollection _resources;
+
+		private AssemblyCollection _assemblyReferences;
+
+		private int _maxAttributeSteps;
+
+		private string _outputAssembly;
+
+		private CompilerOutputType _outputType;
+
+		private bool _debug;
+
+		private bool _ducky;
+
+		private bool _checked;
+
+		private bool _generateInMemory;
+
+		private bool _StdLib;
+
+		private string _keyFile;
+
+		private string _keyContainer;
+
+		private bool _delaySign;
+
+		private ArrayList _libpaths;
+
+		private string _systemDir;
+
+		private Assembly _booAssembly;
 
-		int _maxAttributeSteps;
-		
-		string _outputAssembly;
-		
-		CompilerOutputType _outputType;
-		
-		bool _debug;
-		
-		bool _ducky;
-		
-		bool _generateInMemory;
-		
-		bool _StdLib;
-		
-		string _keyFile;
-		
-		string _keyContainer;
-		
-		bool _delaySign;
-		
-		ArrayList _libpaths;
-		
-		string _systemDir;
-		
-		Assembly _booAssembly;
-		
 		public readonly TraceSwitch TraceSwitch = new TraceSwitch("booc", "boo compiler");
-		
-		public CompilerParameters(): this(true)
+
+		public CompilerParameters()
+			: this(true)
 		{
 		}
-		
-		public CompilerParameters(bool load_default_references)
+
+		public CompilerParameters(bool loadDefaultReferences)
 		{
 			_libpaths = new ArrayList();
 			_systemDir = GetSystemDir();
 			_libpaths.Add(_systemDir);
 			_libpaths.Add(Directory.GetCurrentDirectory());
-			
+
 			_pipeline = null;
 			_input = new CompilerInputCollection();
 			_resources = new CompilerResourceCollection();
 			_assemblyReferences = new AssemblyCollection();
-			
+
 			_maxAttributeSteps = 2;
 			_outputAssembly = string.Empty;
 			_outputType = CompilerOutputType.ConsoleApplication;
 			_outputWriter = System.Console.Out;
 			_debug = true;
+			_checked = true;
 			_generateInMemory = true;
 			_StdLib = true;
-			
+
 			_delaySign = false;
-			
-			if (load_default_references) LoadDefaultReferences();
+
+			if (loadDefaultReferences) LoadDefaultReferences();
 		}
-		
+
 		public void LoadDefaultReferences()
 		{
 			//mscorlib
@@ -128,36 +130,17 @@ namespace Boo.Lang.Compiler
 			_assemblyReferences.Add(_booAssembly);
 			//boo.lang.compiler.dll
 			_assemblyReferences.Add(GetType().Assembly);
-			
+
 			if (TraceSwitch.TraceInfo)
 			{
-				Trace.WriteLine("BOO LANG DLL: "+_booAssembly.Location);
-				Trace.WriteLine("BOO COMPILER DLL: "+GetType().Assembly.Location);
+				Trace.WriteLine("BOO LANG DLL: " + _booAssembly.Location);
+				Trace.WriteLine("BOO COMPILER DLL: " + GetType().Assembly.Location);
 			}
 		}
-		
-		static public bool NET_2_0
-		{
-			get
-			{
-				return _NET_2_0;
-			}
-		}
-		
-		static public bool NET_1_1
-		{
-			get
-			{
-				return _NET_1_1;
-			}
-		}
-		
+
 		public Assembly BooAssembly
 		{
-			get
-			{
-				return _booAssembly;
-			}
+			get { return _booAssembly; }
 			set
 			{
 				if (value != null)
@@ -168,12 +151,12 @@ namespace Boo.Lang.Compiler
 				}
 			}
 		}
-		
+
 		public Assembly FindAssembly(string name)
 		{
 			return _assemblyReferences.Find(name);
 		}
-		
+
 		public void AddAssembly(Assembly asm)
 		{
 			if (asm != null)
@@ -181,206 +164,201 @@ namespace Boo.Lang.Compiler
 				_assemblyReferences.Add(asm);
 			}
 		}
-		
-		public Assembly LoadAssembly (string assembly)
+
+		public Assembly LoadAssembly(string assembly)
 		{
 			return LoadAssembly(assembly, true);
 		}
-		
-		public Assembly LoadAssembly (string assembly, bool throw_errors)
+
+		public Assembly LoadAssembly(string assembly, bool throwOnError)
 		{
 			if (TraceSwitch.TraceInfo)
 			{
-				Trace.WriteLine("ATTEMPTING LOADASSEMBLY: "+assembly);
+				Trace.WriteLine("ATTEMPTING LOADASSEMBLY: " + assembly);
 			}
-			
+
 			Assembly a = null;
-			
-			try 
+			try
 			{
-				char[] path_chars = { '/', '\\' };
-				
-				if (assembly.IndexOfAny(path_chars) != -1)
+				if (assembly.IndexOfAny(new char[] {'/', '\\'}) != -1)
 				{
 					//nant passes full path to gac dlls, which compiler doesn't like:
-					if (assembly.ToLower().StartsWith(_systemDir.ToLower()))
+					//if (assembly.ToLower().StartsWith(_systemDir.ToLower()))
 					{
-						return LoadAssemblyFromGac(Path.GetFileName(assembly), throw_errors);
+						//return LoadAssemblyFromGac(Path.GetFileName(assembly));
 					}
-					else //load using path
+					//else //load using path  
 					{
 						a = Assembly.LoadFrom(assembly);
 					}
 				}
 				else
 				{
-					a = LoadAssemblyFromGac(assembly, throw_errors);
+					a = LoadAssemblyFromGac(assembly);
 				}
 			}
 			catch (FileNotFoundException /*ignored*/)
 			{
-				return LoadAssemblyFromLibPaths(assembly, throw_errors);
+				return LoadAssemblyFromLibPaths(assembly, throwOnError);
 			}
-			catch (BadImageFormatException f)
+			catch (BadImageFormatException e)
 			{
-				if (throw_errors)
+				if (throwOnError)
 				{
 					throw new ApplicationException(Boo.Lang.ResourceManager.Format(
-						"BooC.BadFormat", 
-						f.FusionLog));
-				}
-			} 
-			catch (FileLoadException f)
-			{
-				if (throw_errors)
-				{
-					throw new ApplicationException(Boo.Lang.ResourceManager.Format(
-						"BooC.UnableToLoadAssembly", 
-						f.FusionLog));
-				}
-			} 
-			catch (ArgumentNullException)
-			{
-				if (throw_errors)
-				{
-					throw new ApplicationException(Boo.Lang.ResourceManager.Format(
-						"BooC.NullAssembly"));
+					                               	"BooC.BadFormat",
+					                               	e.FusionLog), e);
 				}
 			}
-			if (a==null)
+			catch (FileLoadException e)
 			{
-				return LoadAssemblyFromLibPaths(assembly, throw_errors);
+				if (throwOnError)
+				{
+					throw new ApplicationException(Boo.Lang.ResourceManager.Format(
+					                               	"BooC.UnableToLoadAssembly",
+					                               	e.FusionLog), e);
+				}
+			}
+			catch (ArgumentNullException e)
+			{
+				if (throwOnError)
+				{
+					throw new ApplicationException(Boo.Lang.ResourceManager.Format(
+					                               	"BooC.NullAssembly"), e);
+				}
+			}
+			if (a == null)
+			{
+				return LoadAssemblyFromLibPaths(assembly, throwOnError);
 			}
 			return a;
 		}
-		
-		private Assembly LoadAssemblyFromLibPaths(string assembly, bool throw_errors)
+
+		private Assembly LoadAssemblyFromLibPaths(string assembly, bool throwOnError)
 		{
 			Assembly a = null;
-			string total_log = "";
+			string fullLog = "";
 			foreach (string dir in _libpaths)
 			{
 				string full_path = Path.Combine(dir, assembly);
-				if (!assembly.EndsWith(".dll") && !assembly.EndsWith(".exe"))
+				FileInfo file = new FileInfo(full_path);
+				if (!_validFileExtensions.Contains(file.Extension.ToLower()))
 					full_path += ".dll";
 
-				try 
+				try
 				{
 					a = Assembly.LoadFrom(full_path);
 					if (a != null)
 					{
 						return a;
 					}
-				} 
+				}
 				catch (FileNotFoundException ff)
 				{
-					total_log += ff.FusionLog;
+					fullLog += ff.FusionLog;
 					continue;
 				}
 			}
-			if (throw_errors)
+			if (throwOnError)
 			{
 				throw new ApplicationException(Boo.Lang.ResourceManager.Format(
-					"BooC.CannotFindAssembly", 
-					assembly)); 
-					//assembly, total_log)); //total_log contains the fusion log
+				                               	"BooC.CannotFindAssembly",
+				                               	assembly));
+				//assembly, total_log)); //total_log contains the fusion log
 			}
 			return a;
 		}
-		
-		private Assembly LoadAssemblyFromGac(string assembly, bool throw_errors)
+
+		private Assembly LoadAssemblyFromGac(string assemblyName)
 		{
-			Assembly a = null;
-			string ass = assembly;
-			if (ass.EndsWith(".dll") || ass.EndsWith(".exe"))
-				ass = ass.Substring(0, ass.Length - 4);
-			
-			a = Assembly.LoadWithPartialName(ass);
-			if (a==null)
-			{
-				a = Assembly.Load(ass);
-			}
-			return a;
+			assemblyName = NormalizeAssemblyName(assemblyName);
+			Assembly assembly = Assembly.LoadWithPartialName(assemblyName);
+			if (assembly != null) return assembly;
+			return Assembly.Load(assemblyName);
 		}
-		
+
+		private static string NormalizeAssemblyName(string assembly)
+		{
+			if (assembly.EndsWith(".dll") || assembly.EndsWith(".exe"))
+			{
+				assembly = assembly.Substring(0, assembly.Length - 4);
+			}
+			return assembly;
+		}
+
+		public void LoadReferencesFromPackage(string package)
+		{
+			string[] libs = Regex.Split(pkgconfig(package), @"\-r\:", RegexOptions.CultureInvariant);
+			foreach (string r in libs)
+			{
+				string reference = r.Trim();
+				if (reference.Length == 0) continue;
+				Trace.WriteLine("LOADING REFERENCE FROM PKGCONFIG '" + package + "' : " + reference);
+				References.Add(LoadAssembly(reference));
+			}
+		}
+
+		private static string pkgconfig(string package)
+		{
+#if NO_SYSTEM_DLL
+	        throw new System.NotSupportedException();
+#else
+			Process process;
+			try
+			{
+				process = Builtins.shellp("pkg-config", string.Format("--libs {0}", package));
+			}
+			catch (Exception e)
+			{
+				throw new ApplicationException(Boo.Lang.ResourceManager.GetString("BooC.PkgConfigNotFound"), e);
+			}
+			process.WaitForExit();
+			if (process.ExitCode != 0)
+			{
+				throw new ApplicationException(
+					Boo.Lang.ResourceManager.Format("BooC.PkgConfigReportedErrors", process.StandardError.ReadToEnd()));
+			}
+			return process.StandardOutput.ReadToEnd();
+#endif
+		}
+
 		private string GetSystemDir()
 		{
-			Assembly[] assemblies = AppDomain.CurrentDomain.GetAssemblies();
-
-			foreach (Assembly a in assemblies)
-			{
-				string codebase;
-				try
-				{
-					codebase = a.Location;
-				}
-				catch (Exception /*ignored*/) //dynamic assembly, ignore
-				{
-					continue;
-				}
-				string fn = System.IO.Path.GetFileName(codebase);
-				if (fn == "corlib.dll" || fn == "mscorlib.dll")
-				{
-					return codebase.Substring(0, codebase.LastIndexOf(Path.DirectorySeparatorChar));
-				}
-			}
-			throw new ApplicationException(Boo.Lang.ResourceManager.Format(
-						"BooC.NoSystemPath"));
+			return Path.GetDirectoryName(typeof(string).Assembly.Location);
 		}
-		
+
 		/// <summary>
 		/// Max number of steps for the resolution of AST attributes.		
 		/// </summary>
 		public int MaxAttributeSteps
 		{
-			get
-			{
-				return _maxAttributeSteps;
-			}
+			get { return _maxAttributeSteps; }
 
-			set
-			{
-				_maxAttributeSteps = value;
-			}
+			set { _maxAttributeSteps = value; }
 		}
 
 		public CompilerInputCollection Input
 		{
-			get
-			{
-				return _input;
-			}
+			get { return _input; }
 		}
-		
+
 		public ArrayList LibPaths
 		{
-			get
-			{
-				return _libpaths;
-			}
+			get { return _libpaths; }
 		}
-		
+
 		public CompilerResourceCollection Resources
 		{
-			get
-			{
-				return _resources;
-			}
+			get { return _resources; }
 		}
 
 		public AssemblyCollection References
 		{
-			get
-			{
-				return _assemblyReferences;
-			}
-			
+			get { return _assemblyReferences; }
+
 			set
 			{
-				if (null == value)
-				{
-					throw new ArgumentNullException("References");
-				}
+				if (null == value) throw new ArgumentNullException("References");
 				_assemblyReferences = value;
 			}
 		}
@@ -390,92 +368,55 @@ namespace Boo.Lang.Compiler
 		/// </summary>
 		public CompilerPipeline Pipeline
 		{
-			get
-			{
-				return _pipeline;
-			}
-			
-			set
-			{
-				_pipeline = value;
-			}
+			get { return _pipeline; }
+
+			set { _pipeline = value; }
 		}
-		
+
 		/// <summary>
 		/// The name (full or partial) for the file
 		/// that should receive the resulting assembly.
 		/// </summary>
 		public string OutputAssembly
 		{
-			get
-			{
-				return _outputAssembly;
-			}
-			
+			get { return _outputAssembly; }
+
 			set
 			{
-				if (null == value)
-				{
-					throw new ArgumentNullException("OutputAssembly");
-				}
-				if (0 == value.Length)
-				{
-					throw new ArgumentException("OutputAssembly");
-				}
+				if (string.IsNullOrEmpty(value)) throw new ArgumentNullException("OutputAssembly");
 				_outputAssembly = value;
 			}
 		}
-		
+
 		/// <summary>
 		/// Type and execution subsystem for the generated portable
 		/// executable file.
 		/// </summary>
 		public CompilerOutputType OutputType
 		{
-			get
-			{
-				return _outputType;
-			}
-			
-			set
-			{
-				_outputType = value;
-			}
+			get { return _outputType; }
+
+			set { _outputType = value; }
 		}
-		
+
 		public bool GenerateInMemory
 		{
-			get
-			{
-				return _generateInMemory;
-			}
-			
-			set
-			{
-				_generateInMemory = value;
-			}
+			get { return _generateInMemory; }
+
+			set { _generateInMemory = value; }
 		}
-		
+
 		public bool StdLib
 		{
-			get
-			{
-				return _StdLib;
-			}
-			
-			set
-			{
-				_StdLib = value;
-			}
+			get { return _StdLib; }
+
+			set { _StdLib = value; }
 		}
-		
+
 		public TextWriter OutputWriter
 		{
-			get
-			{
-				return _outputWriter;
-			}
-			
+			get { return _outputWriter; }
+
 			set
 			{
 				if (null == value)
@@ -485,73 +426,50 @@ namespace Boo.Lang.Compiler
 				_outputWriter = value;
 			}
 		}
-		
+
 		public bool Debug
 		{
-			get
-			{
-				return _debug;
-			}
-			
-			set
-			{
-				_debug = value;
-			}
+			get { return _debug; }
+
+			set { _debug = value; }
 		}
-		
+
 		/// <summary>
 		/// Use duck instead of object as the most generic type.
 		/// </summary>
 		public bool Ducky
 		{
-			get
-			{
-				return _ducky;
-			}
-			
-			set
-			{
-				_ducky = value;
-			}
+			get { return _ducky; }
+
+			set { _ducky = value; }
 		}
-		
+
+		public bool Checked
+		{
+			get { return _checked; }
+
+			set { _checked = value; }
+		}
+
 		public string KeyFile
 		{
-			get
-			{
-				return _keyFile;
-			}
-			
-			set
-			{
-				_keyFile = value;
-			}
+			get { return _keyFile; }
+
+			set { _keyFile = value; }
 		}
-		
+
 		public string KeyContainer
 		{
-			get
-			{
-				return _keyContainer;
-			}
-			
-			set
-			{
-				_keyContainer = value;
-			}
+			get { return _keyContainer; }
+
+			set { _keyContainer = value; }
 		}
-		
+
 		public bool DelaySign
 		{
-			get
-			{
-				return _delaySign;
-			}
-			
-			set
-			{
-				_delaySign = value;
-			}
+			get { return _delaySign; }
+
+			set { _delaySign = value; }
 		}
 	}
-}
+}
\ No newline at end of file
