@@ -98,5 +98,15 @@ namespace Boo.Lang.Compiler
 		{
 			return new CompilerWarning("BCW0010", node.LexicalInfo);
 		}
+
+		public static CompilerWarning AbstractMemberNotImplementedStubCreated(Node node, string typeName, string memberName)
+		{
+			return new CompilerWarning("BCW0011", node.LexicalInfo, typeName, memberName);
+		}
+		
+		public static CompilerWarning Obsolete(Node node, string memberName, string message)
+		{
+			return new CompilerWarning("BCW0012", node.LexicalInfo, memberName, message);
+		}
 	}
 }
