@@ -26,6 +26,8 @@
 // THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #endregion
 
+using System;
+
 namespace Boo.Lang.Compiler.Pipelines
 {
 	using Boo.Lang.Compiler.Steps;
@@ -37,8 +39,11 @@ namespace Boo.Lang.Compiler.Pipelines
 			Add(new UnfoldConstants());
 			Add(new OptimizeIterationStatements());
 
+			Add(new BranchChecking());
+
 			Add(new CheckIdentifiers());
 			Add(new StricterErrorChecking());
+		    Add(new CheckObsoleteUsage());
 
 			Add(new ExpandDuckTypedExpressions());
 
