@@ -67,7 +67,6 @@ namespace Boo.Lang.Compiler.Steps
 		{
 			CantBeMarkedTransient(node);
 			CantBeMarkedPartial(node);
-			CheckExtensionSemantics(node);
 			CannotReturnValue(node);
 		}
 		
@@ -77,7 +76,6 @@ namespace Boo.Lang.Compiler.Steps
 			CantBeMarkedTransient(node);
 			CantBeMarkedPartial(node);
 			CheckExplicitImpl(node);
-			CheckExtensionSemantics(node);
 			CheckModifierCombination(node);
 		}
 		
@@ -109,11 +107,6 @@ namespace Boo.Lang.Compiler.Steps
 			CheckMemberName(node);
 		}
 		
-		override public void LeaveGivenStatement(GivenStatement node)
-		{
-			NotImplemented(node, "given");
-		}
-		
 		override public void LeaveTryStatement(TryStatement node)
 		{
 			if (node.EnsureBlock == null && node.ExceptionHandlers.Count == 0)
@@ -138,7 +131,6 @@ namespace Boo.Lang.Compiler.Steps
 			return (parent.NodeType == NodeType.IfStatement
 				|| parent.NodeType == NodeType.UnlessStatement
 				|| parent.NodeType == NodeType.ConditionalExpression
-				|| parent.NodeType == NodeType.WhenClause
 				|| parent.NodeType == NodeType.StatementModifier
 				|| parent.NodeType == NodeType.ReturnStatement
 				|| parent.NodeType == NodeType.YieldStatement);
@@ -167,13 +159,6 @@ namespace Boo.Lang.Compiler.Steps
 			}
 		}
 
-		void CheckExtensionSemantics(Method node)
-		{
-			if (MethodImplementationFlags.Extension != (node.ImplementationFlags & MethodImplementationFlags.Extension)) return;
-			if (NodeType.Method == node.NodeType && (node.IsStatic || node.DeclaringType is Module)) return;
-			Error(CompilerErrorFactory.InvalidExtensionDefinition(node));
-		}
-		
 		void CantBeMarkedAbstract(TypeMember member)
 		{
 			if (member.IsAbstract)
