@@ -41,28 +41,18 @@ namespace Boo.Lang.Compiler.TypeSystem
 		
 		public BooCodeBuilder(TypeSystemServices tss)
 		{
-			if (null == tss)
-			{
-				throw new ArgumentNullException("tss");
-			}
-			
+			if (null == tss) throw new ArgumentNullException("tss");
 			_tss = tss;
 		}
 		
 		public TypeSystemServices TypeSystemServices
 		{
-			get
-			{
-				return _tss;
-			}
+			get { return _tss; }
 		}
 		
 		public CompilerContext Context
 		{
-			get
-			{
-				return _tss.Context;
-			}
+			get { return _tss.Context; }
 		}
 
 		public int GetFirstParameterIndex(TypeMember member)
@@ -73,22 +63,42 @@ namespace Boo.Lang.Compiler.TypeSystem
 		public Statement CreateFieldAssignment(Field node, Expression initializer)
 		{
 			InternalField fieldEntity = (InternalField)TypeSystem.TypeSystemServices.GetEntity(node);
-			
-			ExpressionStatement stmt = new ExpressionStatement(initializer.LexicalInfo);
-			Expression context = node.IsStatic
-				? (Expression) CreateReference(node.LexicalInfo, fieldEntity.DeclaringType)
-				: CreateSelfReference(fieldEntity.Type);
-			stmt.Expression = this.CreateAssignment(initializer.LexicalInfo,
-				CreateMemberReference(context, fieldEntity),
-				initializer);
-			return stmt;
+			return CreateFieldAssignment(node.LexicalInfo, fieldEntity, initializer);
 		}
 		
-		public Attribute CreateAttribute(IConstructor constructor, Expression arg)
+		public Statement CreateFieldAssignment(LexicalInfo lexicalInfo, IField fieldEntity, Expression initializer)
+		{
+			return new ExpressionStatement(lexicalInfo,
+				CreateFieldAssignmentExpression(fieldEntity, initializer));
+		}
+
+		public Expression CreateFieldAssignmentExpression(IField fieldEntity, Expression initializer)
+		{
+			return CreateAssignment(initializer.LexicalInfo, CreateReference(fieldEntity), initializer);
+		}
+
+		public Attribute CreateAttribute(System.Type type)
+		{
+			return CreateAttribute(_tss.Map(type));
+		}
+		
+		public Attribute CreateAttribute(IType type)
+		{
+			// TODO: check for the existence of a default constructor
+			return CreateAttribute(_tss.GetDefaultConstructor(type));
+		}
+		
+		public Attribute CreateAttribute(IConstructor constructor)
 		{
 			Attribute attribute = new Attribute();
 			attribute.Name = constructor.DeclaringType.FullName;
 			attribute.Entity = constructor;
+			return attribute;
+		}
+		
+		public Attribute CreateAttribute(IConstructor constructor, Expression arg)
+		{
+			Attribute attribute = CreateAttribute(constructor);
 			attribute.Arguments.Add(arg);
 			return attribute;
 		}
@@ -152,11 +162,6 @@ namespace Boo.Lang.Compiler.TypeSystem
 			return CreateTypeofExpression(_tss.Map(type));
 		}
 		
-		public InternalLabel CreateLabelStatement(Node sourceNode, string name)
-		{
-			return new InternalLabel(new LabelStatement(sourceNode.LexicalInfo, name));
-		}
-		
 		public ReferenceExpression CreateLabelReference(LabelStatement label)
 		{
 			ReferenceExpression reference = new ReferenceExpression(label.LexicalInfo, label.Name);
@@ -164,6 +169,12 @@ namespace Boo.Lang.Compiler.TypeSystem
 			return reference;
 		}
 		
+		public Statement CreateSwitch(LexicalInfo li, Expression offset, IEnumerable labels)
+		{
+			offset.LexicalInfo = li;
+			return CreateSwitch(offset, labels);
+		}
+		
 		public Statement CreateSwitch(Expression offset, IEnumerable labels)
 		{
 			MethodInvocationExpression sw = CreateBuiltinInvocation(offset.LexicalInfo, BuiltinFunction.Switch);
@@ -192,7 +203,9 @@ namespace Boo.Lang.Compiler.TypeSystem
 		
 		public MethodInvocationExpression CreatePropertyGet(Expression target, IProperty property)
 		{
-			return CreateMethodInvocation(target, property.GetGetMethod());
+			return property.IsExtension
+				? CreateMethodInvocation(property.GetGetMethod(), target)
+				: CreateMethodInvocation(target, property.GetGetMethod());
 		}
 		
 		public MethodInvocationExpression CreatePropertySet(Expression target, IProperty property, Expression value)
@@ -205,6 +218,11 @@ namespace Boo.Lang.Compiler.TypeSystem
 			return CreateMethodInvocation(_tss.Map(staticMethod), arg);
 		}
 
+		public MethodInvocationExpression CreateMethodInvocation(MethodInfo staticMethod, Expression arg0, Expression arg1)
+		{
+			return CreateMethodInvocation(_tss.Map(staticMethod), arg0, arg1);
+		}
+
 		public MethodInvocationExpression CreateMethodInvocation(LexicalInfo li, Expression target, IMethod tag, Expression arg)
 		{
 			MethodInvocationExpression mie = CreateMethodInvocation(target, tag, arg);
@@ -225,6 +243,13 @@ namespace Boo.Lang.Compiler.TypeSystem
 			mie.Arguments.Add(arg2);
 			return mie;
 		}
+
+		public MethodInvocationExpression CreateMethodInvocation(LexicalInfo li, IMethod staticMethod, Expression arg0, Expression arg1)
+		{
+			MethodInvocationExpression expression = CreateMethodInvocation(staticMethod, arg0, arg1);
+			expression.LexicalInfo = li;
+			return expression;
+		}
 		
 		public MethodInvocationExpression CreateMethodInvocation(IMethod staticMethod, Expression arg0, Expression arg1)
 		{
@@ -232,7 +257,14 @@ namespace Boo.Lang.Compiler.TypeSystem
 			mie.Arguments.Add(arg1);
 			return mie;
 		}
-		
+
+		public MethodInvocationExpression CreateMethodInvocation(LexicalInfo li, IMethod staticMethod, Expression arg0, Expression arg1, Expression arg2)
+		{
+			MethodInvocationExpression expression = CreateMethodInvocation(staticMethod, arg0, arg1, arg2);
+			expression.LexicalInfo = li;
+			return expression;
+		}
+
 		public MethodInvocationExpression CreateMethodInvocation(IMethod staticMethod, Expression arg0, Expression arg1, Expression arg2)
 		{
 			MethodInvocationExpression mie = CreateMethodInvocation(staticMethod, arg0, arg1);
@@ -248,12 +280,11 @@ namespace Boo.Lang.Compiler.TypeSystem
 			return mie;
 		}
 		
-		public MethodInvocationExpression CreateMethodInvocation(IMethod staticMethod)
+		public MethodInvocationExpression CreateMethodInvocation(IMethod method)
 		{
 			MethodInvocationExpression mie = new MethodInvocationExpression();
-			mie.Target = new ReferenceExpression(staticMethod.FullName);
-			mie.Target.Entity = staticMethod;
-			mie.ExpressionType = staticMethod.ReturnType;
+			mie.Target = CreateMemberReference(method);
+			mie.ExpressionType = method.ReturnType;
 			return mie;
 		}
 		
@@ -261,7 +292,19 @@ namespace Boo.Lang.Compiler.TypeSystem
 		{
 			return CreateTypeReference(_tss.Map(type));
 		}
-		
+
+		public TypeReference CreateTypeReference(LexicalInfo li, Type type)
+		{
+			return CreateTypeReference(li, _tss.Map(type));
+		}
+
+		public TypeReference CreateTypeReference(LexicalInfo li, IType type)
+		{
+			TypeReference reference = CreateTypeReference(type);
+			reference.LexicalInfo = li;
+			return reference;
+		}
+
 		public TypeReference CreateTypeReference(IType tag)
 		{
 			TypeReference typeReference = null;
@@ -349,20 +392,19 @@ namespace Boo.Lang.Compiler.TypeSystem
 
         public MemberReferenceExpression CreateMemberReference(IMember member)
 		{
-			IType declaringType = member.DeclaringType;
-			
-			Expression target = null;
-			if (member.IsStatic)
-			{
-				target = CreateReference(declaringType);
-			}
-			else
-			{
-				target = CreateSelfReference(declaringType);
-			}
+        	Expression target = member.IsStatic
+				? (Expression)CreateReference(member.DeclaringType)
+				: (Expression)CreateSelfReference(member.DeclaringType);
 			return CreateMemberReference(target, member);
 		}
-		
+
+		public MemberReferenceExpression CreateMemberReference(LexicalInfo li, Expression target, IMember member)
+		{
+			MemberReferenceExpression expression = CreateMemberReference(target, member);
+			expression.LexicalInfo = li;
+			return expression;
+		}
+
 		public MemberReferenceExpression CreateMemberReference(Expression target, IMember member)
 		{
 			MemberReferenceExpression reference = new MemberReferenceExpression(target.LexicalInfo);
@@ -372,12 +414,19 @@ namespace Boo.Lang.Compiler.TypeSystem
 			reference.ExpressionType = member.Type;
 			return reference;
 		}
-		
-		public MethodInvocationExpression CreateMethodInvocation(Expression target, IMethod tag)
+
+		public MethodInvocationExpression CreateMethodInvocation(LexicalInfo li, Expression target, IMethod entity)
+		{
+			MethodInvocationExpression expression = CreateMethodInvocation(target, entity);
+			expression.LexicalInfo = li;
+			return expression;
+		}
+
+		public MethodInvocationExpression CreateMethodInvocation(Expression target, IMethod entity)
 		{
 			MethodInvocationExpression mie = new MethodInvocationExpression(target.LexicalInfo);
-			mie.Target = CreateMemberReference(target, tag);			
-			mie.ExpressionType = tag.ReturnType;			
+			mie.Target = CreateMemberReference(target, entity);			
+			mie.ExpressionType = entity.ReturnType;
 			return mie;			
 		}
 		
@@ -397,6 +446,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 		{
 			ReferenceExpression reference = new ReferenceExpression(type.FullName);
 			reference.Entity = type;
+			reference.IsSynthetic = true;
 			return reference;
 		}
 		
@@ -588,6 +638,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 			Constructor constructor = new Constructor();
 			constructor.Modifiers = modifiers;
 			constructor.Entity = new InternalConstructor(_tss, constructor);
+			constructor.IsSynthetic = true;
 			return constructor;
 		}
 		
@@ -629,14 +680,14 @@ namespace Boo.Lang.Compiler.TypeSystem
 			return mie;
 		}
 		
-		public Statement CreateSuperConstructorInvocation(IType baseType)
+		public ExpressionStatement CreateSuperConstructorInvocation(IType baseType)
 		{			
 			IConstructor defaultConstructor = _tss.GetDefaultConstructor(baseType);
 			Debug.Assert(null != defaultConstructor);
 			return CreateSuperConstructorInvocation(defaultConstructor);
 		}
 		
-		public Statement CreateSuperConstructorInvocation(IConstructor defaultConstructor)
+		public ExpressionStatement CreateSuperConstructorInvocation(IConstructor defaultConstructor)
 		{			
 			MethodInvocationExpression call = new MethodInvocationExpression(new SuperLiteralExpression());			
 			call.Target.Entity = defaultConstructor;
@@ -682,6 +733,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 			method.Modifiers = modifiers;
 			method.ReturnType = returnType;
 			method.Entity = new InternalMethod(_tss, method);
+			method.IsSynthetic = true;
 			return method;
 		}
 		
@@ -691,6 +743,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 			property.Modifiers = TypeMemberModifiers.Public;
 			property.Type = CreateTypeReference(type);
 			property.Entity = new InternalProperty(_tss, property);
+			property.IsSynthetic = true;
 			return property;
 		}
 		
@@ -701,6 +754,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 			field.Name = name;
 			field.Type = CreateTypeReference(type);
 			field.Entity = new InternalField(field);
+			field.IsSynthetic = true;
 			return field;
 		}
 		
@@ -738,10 +792,16 @@ namespace Boo.Lang.Compiler.TypeSystem
 		}
 		
 		public Method CreateAbstractMethod(LexicalInfo lexicalInfo, IMethod baseMethod)
+		{
+			return CreateMethodFromPrototype(lexicalInfo, baseMethod, TypeMemberModifiers.Public | TypeMemberModifiers.Abstract);
+		}
+		
+		public Method CreateMethodFromPrototype(LexicalInfo lexicalInfo, IMethod baseMethod, TypeMemberModifiers newModifiers)
 		{
 			Method method = new Method(lexicalInfo);
 			method.Name = baseMethod.Name;
-			method.Modifiers = TypeMemberModifiers.Public | TypeMemberModifiers.Abstract;
+			method.Modifiers = newModifiers;
+			method.IsSynthetic = true;
 			
 			IParameter[] parameters = baseMethod.GetParameters();
 			for (int i=0; i<parameters.Length; ++i)
@@ -827,5 +887,39 @@ namespace Boo.Lang.Compiler.TypeSystem
 				parameter.Entity = new InternalParameter(parameter, i + delta);
 			}
 		}
+
+		public InternalLabel CreateLabel(Node sourceNode, string name)
+		{
+			return new InternalLabel(new LabelStatement(sourceNode.LexicalInfo, name));
+		}
+		
+		
+		public TypeMember CreateStub(IMember member)
+		{
+			IMethod md = (member as IMethod);
+			if (null == md) return null;				
+				
+			Method m = CreateVirtualMethod(md.Name, md.ReturnType);
+			int idx = 0;
+			foreach (IParameter param in md.GetParameters()) {
+				m.Parameters.Add(
+					CreateParameterDeclaration(idx,
+											param.Name,
+											param.Type,
+											param.IsByRef));
+				idx++;
+			}
+			
+			MethodInvocationExpression x = new MethodInvocationExpression();
+			x.Target = new MemberReferenceExpression(
+								new ReferenceExpression("System"),
+								"NotImplementedException");
+			RaiseStatement rs = new RaiseStatement(x);
+			rs.LexicalInfo = LexicalInfo.Empty;			
+			m.Body.Statements.Insert(0, rs);
+			
+			return m;
+		}
+		
 	}
 }
