@@ -28,86 +28,68 @@
 
 namespace Boo.Lang.Compiler.TypeSystem
 {
-	public class ExternalEvent : IEvent
+	public class ExternalEvent : ExternalEntity<System.Reflection.EventInfo>, IEvent
 	{
-		TypeSystemServices _typeSystemServices;
+	    private IMethod _add;
+
+	    private IMethod _remove;
 		
-		System.Reflection.EventInfo _event;
-		
-		public ExternalEvent(TypeSystemServices tagManager, System.Reflection.EventInfo event_)
+		public ExternalEvent(TypeSystemServices typeSystemServices, System.Reflection.EventInfo event_) : base(typeSystemServices, event_)
 		{
-			_typeSystemServices = tagManager;
-			_event = event_;
 		}
 		
-		public IType DeclaringType
+		public virtual IType DeclaringType
 		{
-			get
-			{
-				return _typeSystemServices.Map(_event.DeclaringType);
-			}
+			get { return _typeSystemServices.Map(_memberInfo.DeclaringType); }
 		}
 		
-		public IMethod GetAddMethod()
+		public virtual IMethod GetAddMethod()
 		{
-			return (IMethod)_typeSystemServices.Map(_event.GetAddMethod());
+            if (null != _add) return _add;
+			return _add = FindAddMethod();
 		}
-		
-		public IMethod GetRemoveMethod()
+
+	    private IMethod FindAddMethod()
+	    {
+	        return _typeSystemServices.Map(_memberInfo.GetAddMethod(true));
+	    }
+
+	    public virtual IMethod GetRemoveMethod()
 		{
-			return (IMethod)_typeSystemServices.Map(_event.GetRemoveMethod());
+            if (null != _remove) return _remove;
+			return _remove = FindRemoveMethod();
 		}
-		
-		public IMethod GetRaiseMethod()
+
+	    private IMethod FindRemoveMethod()
+	    {
+	        return _typeSystemServices.Map(_memberInfo.GetRemoveMethod(true));
+	    }
+
+	    public virtual IMethod GetRaiseMethod()
 		{
-			return (IMethod)_typeSystemServices.Map(_event.GetRaiseMethod());
+			return _typeSystemServices.Map(_memberInfo.GetRaiseMethod(true));
 		}
 		
 		public System.Reflection.EventInfo EventInfo
 		{
-			get
-			{
-				return _event;
-			}
+			get { return _memberInfo; }
 		}
 		
 		public bool IsPublic
 		{
-			get
-			{
-				return _event.GetAddMethod(true).IsPublic;
-			}
+			get { return GetAddMethod().IsPublic; }
 		}
 		
-		public string Name
+		override public EntityType EntityType
+		{
+			get { return EntityType.Event; }
+		}
+		
+		public virtual IType Type
 		{
 			get
 			{
-				return _event.Name;
-			}
-		}
-		
-		public string FullName
-		{
-			get
-			{
-				return _event.DeclaringType.FullName + "." + _event.Name;
-			}
-		}
-		
-		public EntityType EntityType
-		{
-			get
-			{
-				return EntityType.Event;
-			}
-		}
-		
-		public IType Type
-		{
-			get
-			{
-				return _typeSystemServices.Map(_event.EventHandlerType);
+				return _typeSystemServices.Map(_memberInfo.EventHandlerType);
 			}
 		}
 		
@@ -115,7 +97,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 		{
 			get
 			{
-				return _event.GetAddMethod().IsStatic;
+				return GetAddMethod().IsStatic;
 			}
 		}
 
@@ -123,7 +105,8 @@ namespace Boo.Lang.Compiler.TypeSystem
 		{
 			get
 			{
-				return _event.GetAddMethod().IsAbstract;
+				return GetAddMethod().IsAbstract;
+				
 			}
 		}
 
@@ -131,18 +114,13 @@ namespace Boo.Lang.Compiler.TypeSystem
 		{
 			get
 			{
-				return _event.GetAddMethod().IsVirtual;
+				return GetAddMethod().IsVirtual;
 			}
 		}
-		
-		override public string ToString()
-		{
-			return _event.ToString();
-		}
 
-		public bool IsDuckTyped
+		override protected System.Type MemberType
 		{
-			get { return false; }
+			get { return _memberInfo.EventHandlerType;  }
 		}
 	}
 }
