@@ -31,58 +31,52 @@ namespace Boo.Lang.Compiler.TypeSystem
 	using System;
 	using System.Reflection;
 
-	public class ExternalMethod : IMethod
+	public class ExternalMethod : ExternalEntity<MethodBase>, IMethod
 	{
-		protected TypeSystemServices _typeSystemServices;
-		
-		MethodBase _mi;
-		
 		IParameter[] _parameters;
 		
 		ICallableType _type;
 
+		// TODO: replace by bool?
 		int _acceptVarArgs = -1;
 
-		int _isDuckTyped = -1;
-
 		int _isExtension = -1;
 		
 		int _isPInvoke = -1;
+
+		private int _isMeta = -1;
 		
-		internal ExternalMethod(TypeSystemServices manager, MethodBase mi)
+		internal ExternalMethod(TypeSystemServices manager, MethodBase mi) : base(manager, mi)
 		{
-			_typeSystemServices = manager;
-			_mi = mi;
 		}
 
+		public bool IsMeta
+		{
+			get
+			{
+				if (-1 == _isMeta)
+				{
+					_isMeta = IsStatic && MetadataUtil.IsAttributeDefined(_memberInfo, typeof(Boo.Lang.Compiler.MetaProgramming.MetaAttribute))
+					          	? 1
+					          	: 0;
+				}
+				return _isMeta == 1;
+			}
+		}
+		
 		public bool IsExtension
 		{
 			get
 			{
 				if (-1 == _isExtension)
 				{
-					_isExtension = IsStatic && MetadataUtil.IsAttributeDefined(_mi,  Types.ExtensionAttribute)
+					_isExtension = IsStatic && MetadataUtil.IsAttributeDefined(_memberInfo,  Types.ExtensionAttribute)
 						? 1
 						: 0;
 				}
 				return 1 == _isExtension;
 			}
 		}
-
-		public bool IsDuckTyped
-		{
-			get
-			{
-				if (-1 == _isDuckTyped)
-				{
-					_isDuckTyped =
-						!ReturnType.IsValueType && MetadataUtil.IsAttributeDefined(_mi, Types.DuckTypedAttribute)
-						? 1
-						: 0;
-				}
-				return 1 == _isDuckTyped;
-			}
-		}
 		
 		public bool IsPInvoke
 		{
@@ -90,7 +84,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 			{
 				if (-1 == _isPInvoke)
 				{
-					_isPInvoke = IsStatic && MetadataUtil.IsAttributeDefined(_mi,  Types.DllImportAttribute)
+					_isPInvoke = IsStatic && MetadataUtil.IsAttributeDefined(_memberInfo,  Types.DllImportAttribute)
 						? 1
 						: 0;
 				}
@@ -98,11 +92,11 @@ namespace Boo.Lang.Compiler.TypeSystem
 			}
 		}
 		
-		public IType DeclaringType
+		public virtual IType DeclaringType
 		{
 			get
 			{
-				return _typeSystemServices.Map(_mi.DeclaringType);
+				return _typeSystemServices.Map(_memberInfo.DeclaringType);
 			}
 		}
 		
@@ -110,7 +104,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 		{
 			get
 			{
-				return _mi.IsStatic;
+				return _memberInfo.IsStatic;
 			}
 		}
 		
@@ -118,7 +112,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 		{
 			get
 			{
-				return _mi.IsPublic;
+				return _memberInfo.IsPublic;
 			}
 		}
 		
@@ -126,7 +120,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 		{
 			get
 			{
-				return _mi.IsFamily || _mi.IsFamilyOrAssembly;
+				return _memberInfo.IsFamily || _memberInfo.IsFamilyOrAssembly;
 			}
 		}
 
@@ -134,7 +128,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 		{
 			get
 			{
-				return _mi.IsPrivate;
+				return _memberInfo.IsPrivate;
 			}
 		}
 		
@@ -142,7 +136,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 		{
 			get
 			{
-				return _mi.IsAbstract;
+				return _memberInfo.IsAbstract;
 			}
 		}
 
@@ -150,7 +144,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 		{
 			get
 			{
-				return _mi.IsAssembly;
+				return _memberInfo.IsAssembly;
 			}
 		}
 		
@@ -158,7 +152,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 		{
 			get
 			{
-				return _mi.IsVirtual;
+				return _memberInfo.IsVirtual;
 			}
 		}
 		
@@ -166,33 +160,17 @@ namespace Boo.Lang.Compiler.TypeSystem
 		{
 			get
 			{
-				return _mi.IsSpecialName;
+				return _memberInfo.IsSpecialName;
 			}
 		}
 		
-		public string Name
-		{
-			get
-			{
-				return _mi.Name;
-			}
-		}
-		
-		public string FullName
-		{
-			get
-			{
-				return this.DeclaringType + "." + _mi.Name;
-			}
-		}
-
 		public bool AcceptVarArgs
 		{
 			get
 			{
 				if (_acceptVarArgs == -1)
 				{
-					ParameterInfo[] parameters = _mi.GetParameters();
+					ParameterInfo[] parameters = _memberInfo.GetParameters();
 
 					_acceptVarArgs =
 						parameters.Length > 0 && IsParamArray(parameters[parameters.Length-1]) ? 1 : 0;
@@ -211,7 +189,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 		}
 
 		
-		public virtual EntityType EntityType
+		override public EntityType EntityType
 		{
 			get
 			{
@@ -223,11 +201,9 @@ namespace Boo.Lang.Compiler.TypeSystem
 		{
 			get
 			{
-				if (null == _type)
-				{
-					_type = _typeSystemServices.GetCallableType(this);
-				}
-				return _type;
+				if (null != _type) return _type;
+
+                return _type = _typeSystemServices.GetCallableType(this);
 			}
 		}
 		
@@ -239,54 +215,87 @@ namespace Boo.Lang.Compiler.TypeSystem
 			}
 		}
 		
-		public IParameter[] GetParameters()
+		public virtual IParameter[] GetParameters()
 		{
-			if (null == _parameters)
-			{
-				_parameters = _typeSystemServices.Map(_mi.GetParameters());
-			}
-			return _parameters;
+            if (null != _parameters) return _parameters;
+            return _parameters = _typeSystemServices.Map(_memberInfo.GetParameters());
 		}
-		
+
 		public virtual IType ReturnType
 		{
 			get
 			{
-				MethodInfo mi = _mi as MethodInfo;
-				if (null != mi)
-				{
-					return _typeSystemServices.Map(mi.ReturnType);
-				}
+				MethodInfo mi = _memberInfo as MethodInfo;
+				if (null != mi) return _typeSystemServices.Map(mi.ReturnType);
 				return null;
 			}
 		}
-		
+
 		public MethodBase MethodInfo
 		{
-			get
-			{
-				return _mi;
-			}
+			get { return _memberInfo; }
 		}
 		
 		override public bool Equals(object other)
 		{
 			ExternalMethod rhs = other as ExternalMethod;
-			if (null == rhs)
-			{
-				return false;
-			}
-			return _mi.MethodHandle.Value == rhs._mi.MethodHandle.Value;
+			if (null == rhs) return false;
+			return _memberInfo.MethodHandle.Value == rhs._memberInfo.MethodHandle.Value;
 		}
 		
 		override public int GetHashCode()
 		{
-			return _mi.MethodHandle.Value.GetHashCode();
+			return _memberInfo.MethodHandle.Value.GetHashCode();
 		}
 		
 		override public string ToString()
 		{
 			return _typeSystemServices.GetSignature(this);
 		}
+		
+		ExternalGenericMethodInfo _genericMethodDefinitionInfo = null;		
+		public IGenericMethodInfo GenericInfo
+		{
+			get
+			{
+				if (MethodInfo.IsGenericMethodDefinition)
+				{
+					if (_genericMethodDefinitionInfo == null)
+					{
+						_genericMethodDefinitionInfo = 
+							new ExternalGenericMethodInfo(_typeSystemServices, this);
+					}
+					return _genericMethodDefinitionInfo;
+				}
+				return null;
+			}
+		}
+
+		ExternalConstructedMethodInfo _genericMethodInfo = null;
+		public virtual IConstructedMethodInfo ConstructedInfo
+		{
+			get
+			{
+				if (MethodInfo.IsGenericMethod)
+				{
+					if (_genericMethodInfo == null)
+					{
+						_genericMethodInfo = new ExternalConstructedMethodInfo(_typeSystemServices, this);
+					}
+					return _genericMethodInfo;
+				}
+				return null;
+			}
+		}
+
+		protected override Type MemberType
+		{
+			get
+			{
+				MethodInfo mi = _memberInfo as MethodInfo;
+				if (null != mi) return mi.ReturnType;
+				return null;
+			}
+		}
 	}
 }
