@@ -43,10 +43,17 @@ namespace Boo.Lang.Compiler.TypeSystem
 		{
 			if (null == _signature)
 			{
-				_signature = GetInvokeMethod().CallableType.GetSignature();
+				IMethod invoke = GetInvokeMethod();
+				if (null == invoke) return null;
+				_signature = invoke.CallableType.GetSignature();
 			}
 			return _signature;
 		}
+
+		protected override IType CreateConstructedType(IType[] arguments)
+		{
+			return new GenericConstructedCallableType(_typeSystemServices, this, arguments);
+		}
 		
 		public IMethod GetInvokeMethod()
 		{
@@ -65,7 +72,9 @@ namespace Boo.Lang.Compiler.TypeSystem
 		
 		override public string ToString()
 		{
-			return GetSignature().ToString();
+			CallableSignature signature = GetSignature();
+			if (null == signature) return base.ToString();
+			return signature.ToString();
 		}
 	}
 }
