@@ -72,131 +72,127 @@ namespace Boo.Lang.Parser
 		public const int ABSTRACT = 10;
 		public const int AND = 11;
 		public const int AS = 12;
-		public const int AST = 13;
-		public const int BREAK = 14;
-		public const int CONTINUE = 15;
-		public const int CALLABLE = 16;
-		public const int CAST = 17;
-		public const int CHAR = 18;
-		public const int CLASS = 19;
-		public const int CONSTRUCTOR = 20;
-		public const int DEF = 21;
-		public const int DESTRUCTOR = 22;
-		public const int DO = 23;
-		public const int ELIF = 24;
-		public const int ELSE = 25;
-		public const int ENSURE = 26;
-		public const int ENUM = 27;
-		public const int EVENT = 28;
-		public const int EXCEPT = 29;
-		public const int FAILURE = 30;
-		public const int FINAL = 31;
-		public const int FROM = 32;
-		public const int FOR = 33;
-		public const int FALSE = 34;
-		public const int GET = 35;
-		public const int GIVEN = 36;
-		public const int GOTO = 37;
-		public const int IMPORT = 38;
-		public const int INTERFACE = 39;
-		public const int INTERNAL = 40;
-		public const int IS = 41;
-		public const int ISA = 42;
-		public const int IF = 43;
-		public const int IN = 44;
-		public const int NOT = 45;
-		public const int NULL = 46;
-		public const int OF = 47;
-		public const int OR = 48;
-		public const int OTHERWISE = 49;
-		public const int OVERRIDE = 50;
-		public const int PASS = 51;
-		public const int NAMESPACE = 52;
-		public const int PARTIAL = 53;
-		public const int PUBLIC = 54;
-		public const int PROTECTED = 55;
-		public const int PRIVATE = 56;
-		public const int RAISE = 57;
-		public const int REF = 58;
-		public const int RETURN = 59;
-		public const int RETRY = 60;
-		public const int SET = 61;
-		public const int SELF = 62;
-		public const int SUPER = 63;
-		public const int STATIC = 64;
-		public const int SUCCESS = 65;
-		public const int STRUCT = 66;
-		public const int TRY = 67;
-		public const int TRANSIENT = 68;
-		public const int TRUE = 69;
-		public const int TYPEOF = 70;
-		public const int UNLESS = 71;
-		public const int VIRTUAL = 72;
-		public const int WHEN = 73;
-		public const int WHILE = 74;
-		public const int YIELD = 75;
-		public const int TRIPLE_QUOTED_STRING = 76;
-		public const int EOS = 77;
-		public const int DOUBLE_QUOTED_STRING = 78;
-		public const int SINGLE_QUOTED_STRING = 79;
-		public const int ID = 80;
-		public const int LPAREN = 81;
-		public const int RPAREN = 82;
-		public const int ASSIGN = 83;
-		public const int SUBTRACT = 84;
-		public const int LBRACK = 85;
-		public const int COMMA = 86;
-		public const int RBRACK = 87;
-		public const int ASSEMBLY_ATTRIBUTE_BEGIN = 88;
-		public const int DOT = 89;
-		public const int COLON = 90;
-		public const int MULTIPLY = 91;
-		public const int BITWISE_OR = 92;
-		public const int LBRACE = 93;
-		public const int RBRACE = 94;
-		public const int INPLACE_BITWISE_OR = 95;
-		public const int INPLACE_BITWISE_AND = 96;
-		public const int INPLACE_SHIFT_LEFT = 97;
-		public const int INPLACE_SHIFT_RIGHT = 98;
-		public const int CMP_OPERATOR = 99;
-		public const int GREATER_THAN = 100;
-		public const int LESS_THAN = 101;
-		public const int ADD = 102;
-		public const int EXCLUSIVE_OR = 103;
-		public const int DIVISION = 104;
-		public const int MODULUS = 105;
-		public const int BITWISE_AND = 106;
-		public const int SHIFT_LEFT = 107;
-		public const int SHIFT_RIGHT = 108;
-		public const int EXPONENTIATION = 109;
-		public const int INCREMENT = 110;
-		public const int DECREMENT = 111;
-		public const int ONES_COMPLEMENT = 112;
-		public const int INT = 113;
-		public const int LONG = 114;
-		public const int RE_LITERAL = 115;
-		public const int DOUBLE = 116;
-		public const int FLOAT = 117;
-		public const int TIMESPAN = 118;
-		public const int LINE_CONTINUATION = 119;
-		public const int SL_COMMENT = 120;
-		public const int ML_COMMENT = 121;
-		public const int WS = 122;
-		public const int X_RE_LITERAL = 123;
-		public const int NEWLINE = 124;
-		public const int ESCAPED_EXPRESSION = 125;
-		public const int DQS_ESC = 126;
-		public const int SQS_ESC = 127;
-		public const int SESC = 128;
-		public const int RE_CHAR = 129;
-		public const int X_RE_CHAR = 130;
-		public const int RE_ESC = 131;
-		public const int DIGIT_GROUP = 132;
-		public const int REVERSE_DIGIT_GROUP = 133;
-		public const int ID_PREFIX = 134;
-		public const int ID_LETTER = 135;
-		public const int DIGIT = 136;
-		public const int HEXDIGIT = 137;
+		public const int BREAK = 13;
+		public const int CONTINUE = 14;
+		public const int CALLABLE = 15;
+		public const int CAST = 16;
+		public const int CHAR = 17;
+		public const int CLASS = 18;
+		public const int CONSTRUCTOR = 19;
+		public const int DEF = 20;
+		public const int DESTRUCTOR = 21;
+		public const int DO = 22;
+		public const int ELIF = 23;
+		public const int ELSE = 24;
+		public const int ENSURE = 25;
+		public const int ENUM = 26;
+		public const int EVENT = 27;
+		public const int EXCEPT = 28;
+		public const int FINAL = 29;
+		public const int FROM = 30;
+		public const int FOR = 31;
+		public const int FALSE = 32;
+		public const int GET = 33;
+		public const int GOTO = 34;
+		public const int IMPORT = 35;
+		public const int INTERFACE = 36;
+		public const int INTERNAL = 37;
+		public const int IS = 38;
+		public const int ISA = 39;
+		public const int IF = 40;
+		public const int IN = 41;
+		public const int NOT = 42;
+		public const int NULL = 43;
+		public const int OF = 44;
+		public const int OR = 45;
+		public const int OVERRIDE = 46;
+		public const int PASS = 47;
+		public const int NAMESPACE = 48;
+		public const int PARTIAL = 49;
+		public const int PUBLIC = 50;
+		public const int PROTECTED = 51;
+		public const int PRIVATE = 52;
+		public const int RAISE = 53;
+		public const int REF = 54;
+		public const int RETURN = 55;
+		public const int SET = 56;
+		public const int SELF = 57;
+		public const int SUPER = 58;
+		public const int STATIC = 59;
+		public const int STRUCT = 60;
+		public const int TRY = 61;
+		public const int TRANSIENT = 62;
+		public const int TRUE = 63;
+		public const int TYPEOF = 64;
+		public const int UNLESS = 65;
+		public const int VIRTUAL = 66;
+		public const int WHILE = 67;
+		public const int YIELD = 68;
+		public const int TRIPLE_QUOTED_STRING = 69;
+		public const int EOS = 70;
+		public const int DOUBLE_QUOTED_STRING = 71;
+		public const int SINGLE_QUOTED_STRING = 72;
+		public const int ID = 73;
+		public const int LBRACK = 74;
+		public const int RBRACK = 75;
+		public const int LPAREN = 76;
+		public const int RPAREN = 77;
+		public const int ASSIGN = 78;
+		public const int SUBTRACT = 79;
+		public const int COMMA = 80;
+		public const int ASSEMBLY_ATTRIBUTE_BEGIN = 81;
+		public const int SPLICE_BEGIN = 82;
+		public const int DOT = 83;
+		public const int COLON = 84;
+		public const int MULTIPLY = 85;
+		public const int BITWISE_OR = 86;
+		public const int LBRACE = 87;
+		public const int RBRACE = 88;
+		public const int QQ_BEGIN = 89;
+		public const int QQ_END = 90;
+		public const int INPLACE_BITWISE_OR = 91;
+		public const int INPLACE_BITWISE_AND = 92;
+		public const int INPLACE_SHIFT_LEFT = 93;
+		public const int INPLACE_SHIFT_RIGHT = 94;
+		public const int CMP_OPERATOR = 95;
+		public const int GREATER_THAN = 96;
+		public const int LESS_THAN = 97;
+		public const int ADD = 98;
+		public const int EXCLUSIVE_OR = 99;
+		public const int DIVISION = 100;
+		public const int MODULUS = 101;
+		public const int BITWISE_AND = 102;
+		public const int SHIFT_LEFT = 103;
+		public const int SHIFT_RIGHT = 104;
+		public const int EXPONENTIATION = 105;
+		public const int INCREMENT = 106;
+		public const int DECREMENT = 107;
+		public const int ONES_COMPLEMENT = 108;
+		public const int INT = 109;
+		public const int LONG = 110;
+		public const int RE_LITERAL = 111;
+		public const int DOUBLE = 112;
+		public const int FLOAT = 113;
+		public const int TIMESPAN = 114;
+		public const int LINE_CONTINUATION = 115;
+		public const int SL_COMMENT = 116;
+		public const int ML_COMMENT = 117;
+		public const int WS = 118;
+		public const int X_RE_LITERAL = 119;
+		public const int NEWLINE = 120;
+		public const int ESCAPED_EXPRESSION = 121;
+		public const int DQS_ESC = 122;
+		public const int SQS_ESC = 123;
+		public const int SESC = 124;
+		public const int RE_CHAR = 125;
+		public const int X_RE_CHAR = 126;
+		public const int RE_ESC = 127;
+		public const int DIGIT_GROUP = 128;
+		public const int REVERSE_DIGIT_GROUP = 129;
+		public const int ID_PREFIX = 130;
+		public const int ID_LETTER = 131;
+		public const int DIGIT = 132;
+		public const int HEXDIGIT = 133;
 		
 		
 	
@@ -230,72 +226,65 @@ namespace Boo.Lang.Parser
 			caseSensitiveLiterals = true;
 			setCaseSensitive(true);
 			literals = new Hashtable(100, (float) 0.4, null, Comparer.Default);
-			literals.Add("public", 54);
-			literals.Add("namespace", 52);
-			literals.Add("break", 14);
-			literals.Add("while", 74);
-			literals.Add("otherwise", 49);
-			literals.Add("raise", 57);
-			literals.Add("typeof", 70);
+			literals.Add("public", 50);
+			literals.Add("namespace", 48);
+			literals.Add("break", 13);
+			literals.Add("while", 67);
+			literals.Add("raise", 53);
+			literals.Add("typeof", 64);
 			literals.Add("and", 11);
-			literals.Add("failure", 30);
-			literals.Add("not", 45);
-			literals.Add("return", 59);
-			literals.Add("pass", 51);
-			literals.Add("from", 32);
-			literals.Add("null", 46);
-			literals.Add("def", 21);
-			literals.Add("given", 36);
-			literals.Add("protected", 55);
-			literals.Add("ref", 58);
-			literals.Add("retry", 60);
-			literals.Add("when", 73);
-			literals.Add("class", 19);
-			literals.Add("do", 23);
-			literals.Add("except", 29);
-			literals.Add("event", 28);
-			literals.Add("ast", 13);
-			literals.Add("unless", 71);
-			literals.Add("super", 63);
-			literals.Add("set", 61);
-			literals.Add("transient", 68);
-			literals.Add("constructor", 20);
-			literals.Add("interface", 39);
-			literals.Add("of", 47);
-			literals.Add("is", 41);
-			literals.Add("internal", 40);
-			literals.Add("final", 31);
-			literals.Add("yield", 75);
-			literals.Add("or", 48);
-			literals.Add("destructor", 22);
-			literals.Add("if", 43);
-			literals.Add("success", 65);
-			literals.Add("override", 50);
+			literals.Add("not", 42);
+			literals.Add("return", 55);
+			literals.Add("pass", 47);
+			literals.Add("from", 30);
+			literals.Add("null", 43);
+			literals.Add("def", 20);
+			literals.Add("protected", 51);
+			literals.Add("ref", 54);
+			literals.Add("class", 18);
+			literals.Add("do", 22);
+			literals.Add("except", 28);
+			literals.Add("event", 27);
+			literals.Add("unless", 65);
+			literals.Add("super", 58);
+			literals.Add("set", 56);
+			literals.Add("transient", 62);
+			literals.Add("constructor", 19);
+			literals.Add("interface", 36);
+			literals.Add("of", 44);
+			literals.Add("is", 38);
+			literals.Add("internal", 37);
+			literals.Add("final", 29);
+			literals.Add("yield", 68);
+			literals.Add("or", 45);
+			literals.Add("destructor", 21);
+			literals.Add("if", 40);
+			literals.Add("override", 46);
 			literals.Add("as", 12);
-			literals.Add("try", 67);
-			literals.Add("goto", 37);
-			literals.Add("enum", 27);
-			literals.Add("isa", 42);
-			literals.Add("for", 33);
-			literals.Add("char", 18);
-			literals.Add("private", 56);
-			literals.Add("false", 34);
-			literals.Add("static", 64);
+			literals.Add("try", 61);
+			literals.Add("goto", 34);
+			literals.Add("enum", 26);
+			literals.Add("isa", 39);
+			literals.Add("for", 31);
+			literals.Add("char", 17);
+			literals.Add("private", 52);
+			literals.Add("false", 32);
+			literals.Add("static", 59);
 			literals.Add("abstract", 10);
-			literals.Add("partial", 53);
-			literals.Add("callable", 16);
-			literals.Add("get", 35);
-			literals.Add("continue", 15);
-			literals.Add("cast", 17);
-			literals.Add("struct", 66);
-			literals.Add("else", 25);
-			literals.Add("import", 38);
-			literals.Add("elif", 24);
-			literals.Add("in", 44);
-			literals.Add("self", 62);
-			literals.Add("ensure", 26);
-			literals.Add("true", 69);
-			literals.Add("virtual", 72);
+			literals.Add("partial", 49);
+			literals.Add("callable", 15);
+			literals.Add("get", 33);
+			literals.Add("continue", 14);
+			literals.Add("cast", 16);
+			literals.Add("struct", 60);
+			literals.Add("else", 24);
+			literals.Add("import", 35);
+			literals.Add("elif", 23);
+			literals.Add("in", 41);
+			literals.Add("self", 57);
+			literals.Add("ensure", 25);
+			literals.Add("true", 63);
+			literals.Add("virtual", 66);
 		}
 		
 		override public IToken nextToken()			//throws TokenStreamException
@@ -339,12 +328,6 @@ tryAgain:
 							theRetToken = returnToken_;
 							break;
 						}
-						case '|':
-						{
-							mBITWISE_OR(true);
-							theRetToken = returnToken_;
-							break;
-						}
 						case '&':
 						{
 							mBITWISE_AND(true);
@@ -369,12 +352,6 @@ tryAgain:
 							theRetToken = returnToken_;
 							break;
 						}
-						case '[':
-						{
-							mLBRACK(true);
-							theRetToken = returnToken_;
-							break;
-						}
 						case ']':
 						{
 							mRBRACK(true);
@@ -393,6 +370,12 @@ tryAgain:
 							theRetToken = returnToken_;
 							break;
 						}
+						case '$':
+						{
+							mSPLICE_BEGIN(true);
+							theRetToken = returnToken_;
+							break;
+						}
 						case '%':
 						{
 							mMODULUS(true);
@@ -445,6 +428,14 @@ tryAgain:
 								mINPLACE_SHIFT_RIGHT(true);
 								theRetToken = returnToken_;
 							}
+							else if ((cached_LA1=='[') && (cached_LA2=='|')) {
+								mQQ_BEGIN(true);
+								theRetToken = returnToken_;
+							}
+							else if ((cached_LA1=='|') && (cached_LA2==']')) {
+								mQQ_END(true);
+								theRetToken = returnToken_;
+							}
 							else if ((cached_LA1=='+') && (cached_LA2=='+')) {
 								mINCREMENT(true);
 								theRetToken = returnToken_;
@@ -469,6 +460,14 @@ tryAgain:
 								mID(true);
 								theRetToken = returnToken_;
 							}
+							else if ((cached_LA1=='|') && (true)) {
+								mBITWISE_OR(true);
+								theRetToken = returnToken_;
+							}
+							else if ((cached_LA1=='[') && (true)) {
+								mLBRACK(true);
+								theRetToken = returnToken_;
+							}
 							else if ((cached_LA1=='+') && (true)) {
 								mADD(true);
 								theRetToken = returnToken_;
@@ -517,7 +516,7 @@ tryAgain:
 		_ttype = ID;
 		
 		{
-			if ((cached_LA1=='$'||cached_LA1=='?'||cached_LA1=='@'))
+			if ((cached_LA1=='?'||cached_LA1=='@'))
 			{
 				mID_PREFIX(false);
 			}
@@ -564,11 +563,6 @@ _loop4_breakloop:			;
 		
 		switch ( cached_LA1 )
 		{
-		case '$':
-		{
-			match('$');
-			break;
-		}
 		case '@':
 		{
 			match('@');
@@ -1074,11 +1068,11 @@ _loop8_breakloop:					;
 				}
 				default:
 				{
-					goto _loop113_breakloop;
+					goto _loop116_breakloop;
 				}
 				 }
 			}
-_loop113_breakloop:			;
+_loop116_breakloop:			;
 		}    // ( ... )*
 		if (_createToken && (null == _token) && (_ttype != Token.SKIP))
 		{
@@ -1094,7 +1088,7 @@ _loop113_breakloop:			;
 		_ttype = REVERSE_DIGIT_GROUP;
 		
 		{ // ( ... )+
-			int _cnt117=0;
+			int _cnt120=0;
 			for (;;)
 			{
 				if (((cached_LA1 >= '0' && cached_LA1 <= '9')) && ((cached_LA2 >= '0' && cached_LA2 <= '9')) && ((LA(3) >= '0' && LA(3) <= '9')))
@@ -1120,12 +1114,12 @@ _loop113_breakloop:			;
 				}
 				else
 				{
-					if (_cnt117 >= 1) { goto _loop117_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+					if (_cnt120 >= 1) { goto _loop120_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
 				}
 				
-				_cnt117++;
+				_cnt120++;
 			}
-_loop117_breakloop:			;
+_loop120_breakloop:			;
 		}    // ( ... )+
 		if (_createToken && (null == _token) && (_ttype != Token.SKIP))
 		{
@@ -1461,6 +1455,48 @@ _loop117_breakloop:			;
 		returnToken_ = _token;
 	}
 	
+	public void mSPLICE_BEGIN(bool _createToken) //throws RecognitionException, CharStreamException, TokenStreamException
+{
+		int _ttype; IToken _token=null; int _begin=text.Length;
+		_ttype = SPLICE_BEGIN;
+		
+		match('$');
+		if (_createToken && (null == _token) && (_ttype != Token.SKIP))
+		{
+			_token = makeToken(_ttype);
+			_token.setText(text.ToString(_begin, text.Length-_begin));
+		}
+		returnToken_ = _token;
+	}
+	
+	public void mQQ_BEGIN(bool _createToken) //throws RecognitionException, CharStreamException, TokenStreamException
+{
+		int _ttype; IToken _token=null; int _begin=text.Length;
+		_ttype = QQ_BEGIN;
+		
+		match("[|");
+		if (_createToken && (null == _token) && (_ttype != Token.SKIP))
+		{
+			_token = makeToken(_ttype);
+			_token.setText(text.ToString(_begin, text.Length-_begin));
+		}
+		returnToken_ = _token;
+	}
+	
+	public void mQQ_END(bool _createToken) //throws RecognitionException, CharStreamException, TokenStreamException
+{
+		int _ttype; IToken _token=null; int _begin=text.Length;
+		_ttype = QQ_END;
+		
+		match("|]");
+		if (_createToken && (null == _token) && (_ttype != Token.SKIP))
+		{
+			_token = makeToken(_ttype);
+			_token.setText(text.ToString(_begin, text.Length-_begin));
+		}
+		returnToken_ = _token;
+	}
+	
 	public void mINCREMENT(bool _createToken) //throws RecognitionException, CharStreamException, TokenStreamException
 {
 		int _ttype; IToken _token=null; int _begin=text.Length;
@@ -1606,11 +1642,11 @@ _loop117_breakloop:			;
 		int _ttype; IToken _token=null; int _begin=text.Length;
 		_ttype = DIVISION;
 		
-		bool synPredMatched64 = false;
+		bool synPredMatched67 = false;
 		if (((cached_LA1=='/') && (tokenSet_3_.member(cached_LA2)) && (tokenSet_4_.member(LA(3)))))
 		{
-			int _m64 = mark();
-			synPredMatched64 = true;
+			int _m67 = mark();
+			synPredMatched67 = true;
 			inputState.guessing++;
 			try {
 				{
@@ -1619,12 +1655,12 @@ _loop117_breakloop:			;
 			}
 			catch (RecognitionException)
 			{
-				synPredMatched64 = false;
+				synPredMatched67 = false;
 			}
-			rewind(_m64);
+			rewind(_m67);
 			inputState.guessing--;
 		}
-		if ( synPredMatched64 )
+		if ( synPredMatched67 )
 		{
 			mRE_LITERAL(false);
 			if (0==inputState.guessing)
@@ -1668,7 +1704,7 @@ _loop117_breakloop:			;
 		
 		match('/');
 		{ // ( ... )+
-			int _cnt100=0;
+			int _cnt103=0;
 			for (;;)
 			{
 				if ((tokenSet_3_.member(cached_LA1)))
@@ -1677,12 +1713,12 @@ _loop117_breakloop:			;
 				}
 				else
 				{
-					if (_cnt100 >= 1) { goto _loop100_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+					if (_cnt103 >= 1) { goto _loop103_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
 				}
 				
-				_cnt100++;
+				_cnt103++;
 			}
-_loop100_breakloop:			;
+_loop103_breakloop:			;
 		}    // ( ... )+
 		match('/');
 		if (_createToken && (null == _token) && (_ttype != Token.SKIP))
@@ -1880,7 +1916,7 @@ _loop100_breakloop:			;
 		_ttype = WS;
 		
 		{ // ( ... )+
-			int _cnt79=0;
+			int _cnt82=0;
 			for (;;)
 			{
 				switch ( cached_LA1 )
@@ -1915,12 +1951,12 @@ _loop100_breakloop:			;
 				}
 				default:
 				{
-					if (_cnt79 >= 1) { goto _loop79_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+					if (_cnt82 >= 1) { goto _loop82_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
 				}
 				break; }
-				_cnt79++;
+				_cnt82++;
 			}
-_loop79_breakloop:			;
+_loop82_breakloop:			;
 		}    // ( ... )+
 		if (0==inputState.guessing)
 		{
@@ -1957,11 +1993,11 @@ _loop79_breakloop:			;
 				}
 				else
 				{
-					goto _loop83_breakloop;
+					goto _loop86_breakloop;
 				}
 				
 			}
-_loop83_breakloop:			;
+_loop86_breakloop:			;
 		}    // ( ... )*
 		_saveIndex = text.Length;
 		match('\'');
@@ -2368,7 +2404,7 @@ _loop83_breakloop:			;
 			case '8':  case '9':
 			{
 				{ // ( ... )+
-					int _cnt107=0;
+					int _cnt110=0;
 					for (;;)
 					{
 						if (((cached_LA1 >= '0' && cached_LA1 <= '9')) && (tokenSet_4_.member(cached_LA2)) && (true))
@@ -2377,12 +2413,12 @@ _loop83_breakloop:			;
 						}
 						else
 						{
-							if (_cnt107 >= 1) { goto _loop107_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+							if (_cnt110 >= 1) { goto _loop110_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
 						}
 						
-						_cnt107++;
+						_cnt110++;
 					}
-_loop107_breakloop:					;
+_loop110_breakloop:					;
 				}    // ( ... )+
 				break;
 			}
@@ -2499,7 +2535,7 @@ _loop107_breakloop:					;
 	private static long[] mk_tokenSet_0_()
 	{
 		long[] data = new long[3072];
-		data[0]=-9223371968135299072L;
+		data[0]=-9223372036854775808L;
 		data[1]=576460745995190271L;
 		for (int i = 2; i<=1022; i++) { data[i]=-1L; }
 		data[1023]=9223372036854775807L;
