@@ -69,131 +69,127 @@ namespace Boo.Lang.Parser
 		public const int ABSTRACT = 7;
 		public const int AND = 8;
 		public const int AS = 9;
-		public const int AST = 10;
-		public const int BREAK = 11;
-		public const int CONTINUE = 12;
-		public const int CALLABLE = 13;
-		public const int CAST = 14;
-		public const int CHAR = 15;
-		public const int CLASS = 16;
-		public const int CONSTRUCTOR = 17;
-		public const int DEF = 18;
-		public const int DESTRUCTOR = 19;
-		public const int DO = 20;
-		public const int ELIF = 21;
-		public const int ELSE = 22;
-		public const int END = 23;
-		public const int ENSURE = 24;
-		public const int ENUM = 25;
-		public const int EVENT = 26;
-		public const int EXCEPT = 27;
-		public const int FAILURE = 28;
-		public const int FINAL = 29;
-		public const int FROM = 30;
-		public const int FOR = 31;
-		public const int FALSE = 32;
-		public const int GET = 33;
-		public const int GIVEN = 34;
-		public const int GOTO = 35;
-		public const int IMPORT = 36;
-		public const int INTERFACE = 37;
-		public const int INTERNAL = 38;
-		public const int IS = 39;
-		public const int ISA = 40;
-		public const int IF = 41;
-		public const int IN = 42;
-		public const int NOT = 43;
-		public const int NULL = 44;
-		public const int OR = 45;
-		public const int OTHERWISE = 46;
-		public const int OVERRIDE = 47;
-		public const int NAMESPACE = 48;
-		public const int PUBLIC = 49;
-		public const int PROTECTED = 50;
-		public const int PRIVATE = 51;
-		public const int RAISE = 52;
-		public const int REF = 53;
-		public const int RETURN = 54;
-		public const int RETRY = 55;
-		public const int SET = 56;
-		public const int SELF = 57;
-		public const int SUPER = 58;
-		public const int STATIC = 59;
-		public const int SUCCESS = 60;
-		public const int STRUCT = 61;
-		public const int TRY = 62;
-		public const int TRANSIENT = 63;
-		public const int TRUE = 64;
-		public const int TYPEOF = 65;
-		public const int UNLESS = 66;
-		public const int VIRTUAL = 67;
-		public const int PARTIAL = 68;
-		public const int WHEN = 69;
-		public const int WHILE = 70;
-		public const int YIELD = 71;
-		public const int TRIPLE_QUOTED_STRING = 72;
-		public const int EOS = 73;
-		public const int DOUBLE_QUOTED_STRING = 74;
-		public const int SINGLE_QUOTED_STRING = 75;
-		public const int ID = 76;
-		public const int LPAREN = 77;
-		public const int RPAREN = 78;
-		public const int ASSIGN = 79;
-		public const int SUBTRACT = 80;
-		public const int LBRACK = 81;
-		public const int COMMA = 82;
-		public const int RBRACK = 83;
-		public const int ASSEMBLY_ATTRIBUTE_BEGIN = 84;
-		public const int DOT = 85;
-		public const int COLON = 86;
-		public const int MULTIPLY = 87;
-		public const int BITWISE_OR = 88;
-		public const int LBRACE = 89;
-		public const int RBRACE = 90;
-		public const int INPLACE_BITWISE_OR = 91;
-		public const int INPLACE_BITWISE_AND = 92;
-		public const int INPLACE_SHIFT_LEFT = 93;
-		public const int INPLACE_SHIFT_RIGHT = 94;
-		public const int CMP_OPERATOR = 95;
-		public const int GREATER_THAN = 96;
-		public const int LESS_THAN = 97;
-		public const int ADD = 98;
-		public const int EXCLUSIVE_OR = 99;
-		public const int DIVISION = 100;
-		public const int MODULUS = 101;
-		public const int BITWISE_AND = 102;
-		public const int SHIFT_LEFT = 103;
-		public const int SHIFT_RIGHT = 104;
-		public const int EXPONENTIATION = 105;
-		public const int INCREMENT = 106;
-		public const int DECREMENT = 107;
-		public const int ONES_COMPLEMENT = 108;
-		public const int OF = 109;
-		public const int INT = 110;
-		public const int LONG = 111;
-		public const int RE_LITERAL = 112;
-		public const int DOUBLE = 113;
-		public const int FLOAT = 114;
-		public const int TIMESPAN = 115;
-		public const int LINE_CONTINUATION = 116;
-		public const int SL_COMMENT = 117;
-		public const int ML_COMMENT = 118;
-		public const int WS = 119;
-		public const int X_RE_LITERAL = 120;
-		public const int NEWLINE = 121;
-		public const int ESCAPED_EXPRESSION = 122;
-		public const int DQS_ESC = 123;
-		public const int SQS_ESC = 124;
-		public const int SESC = 125;
-		public const int RE_CHAR = 126;
-		public const int X_RE_CHAR = 127;
-		public const int RE_ESC = 128;
-		public const int DIGIT_GROUP = 129;
-		public const int REVERSE_DIGIT_GROUP = 130;
-		public const int ID_PREFIX = 131;
-		public const int ID_LETTER = 132;
-		public const int DIGIT = 133;
-		public const int HEXDIGIT = 134;
+		public const int BREAK = 10;
+		public const int CONTINUE = 11;
+		public const int CALLABLE = 12;
+		public const int CAST = 13;
+		public const int CHAR = 14;
+		public const int CLASS = 15;
+		public const int CONSTRUCTOR = 16;
+		public const int DEF = 17;
+		public const int DESTRUCTOR = 18;
+		public const int DO = 19;
+		public const int ELIF = 20;
+		public const int ELSE = 21;
+		public const int END = 22;
+		public const int ENSURE = 23;
+		public const int ENUM = 24;
+		public const int EVENT = 25;
+		public const int EXCEPT = 26;
+		public const int FINAL = 27;
+		public const int FROM = 28;
+		public const int FOR = 29;
+		public const int FALSE = 30;
+		public const int GET = 31;
+		public const int GOTO = 32;
+		public const int IMPORT = 33;
+		public const int INTERFACE = 34;
+		public const int INTERNAL = 35;
+		public const int IS = 36;
+		public const int ISA = 37;
+		public const int IF = 38;
+		public const int IN = 39;
+		public const int NOT = 40;
+		public const int NULL = 41;
+		public const int OF = 42;
+		public const int OR = 43;
+		public const int OVERRIDE = 44;
+		public const int NAMESPACE = 45;
+		public const int PUBLIC = 46;
+		public const int PROTECTED = 47;
+		public const int PRIVATE = 48;
+		public const int RAISE = 49;
+		public const int REF = 50;
+		public const int RETURN = 51;
+		public const int SET = 52;
+		public const int SELF = 53;
+		public const int SUPER = 54;
+		public const int STATIC = 55;
+		public const int STRUCT = 56;
+		public const int TRY = 57;
+		public const int TRANSIENT = 58;
+		public const int TRUE = 59;
+		public const int TYPEOF = 60;
+		public const int UNLESS = 61;
+		public const int VIRTUAL = 62;
+		public const int PARTIAL = 63;
+		public const int WHILE = 64;
+		public const int YIELD = 65;
+		public const int TRIPLE_QUOTED_STRING = 66;
+		public const int EOS = 67;
+		public const int NEWLINE = 68;
+		public const int DOUBLE_QUOTED_STRING = 69;
+		public const int SINGLE_QUOTED_STRING = 70;
+		public const int ID = 71;
+		public const int LBRACK = 72;
+		public const int RBRACK = 73;
+		public const int LPAREN = 74;
+		public const int RPAREN = 75;
+		public const int ASSIGN = 76;
+		public const int SUBTRACT = 77;
+		public const int COMMA = 78;
+		public const int ASSEMBLY_ATTRIBUTE_BEGIN = 79;
+		public const int DOT = 80;
+		public const int COLON = 81;
+		public const int MULTIPLY = 82;
+		public const int SPLICE_BEGIN = 83;
+		public const int BITWISE_OR = 84;
+		public const int LBRACE = 85;
+		public const int RBRACE = 86;
+		public const int QQ_BEGIN = 87;
+		public const int QQ_END = 88;
+		public const int INPLACE_BITWISE_OR = 89;
+		public const int INPLACE_BITWISE_AND = 90;
+		public const int INPLACE_SHIFT_LEFT = 91;
+		public const int INPLACE_SHIFT_RIGHT = 92;
+		public const int CMP_OPERATOR = 93;
+		public const int GREATER_THAN = 94;
+		public const int LESS_THAN = 95;
+		public const int ADD = 96;
+		public const int EXCLUSIVE_OR = 97;
+		public const int DIVISION = 98;
+		public const int MODULUS = 99;
+		public const int BITWISE_AND = 100;
+		public const int SHIFT_LEFT = 101;
+		public const int SHIFT_RIGHT = 102;
+		public const int EXPONENTIATION = 103;
+		public const int INCREMENT = 104;
+		public const int DECREMENT = 105;
+		public const int ONES_COMPLEMENT = 106;
+		public const int INT = 107;
+		public const int LONG = 108;
+		public const int RE_LITERAL = 109;
+		public const int DOUBLE = 110;
+		public const int FLOAT = 111;
+		public const int TIMESPAN = 112;
+		public const int LINE_CONTINUATION = 113;
+		public const int SL_COMMENT = 114;
+		public const int ML_COMMENT = 115;
+		public const int WS = 116;
+		public const int X_RE_LITERAL = 117;
+		public const int ESCAPED_EXPRESSION = 118;
+		public const int DQS_ESC = 119;
+		public const int SQS_ESC = 120;
+		public const int SESC = 121;
+		public const int RE_CHAR = 122;
+		public const int X_RE_CHAR = 123;
+		public const int RE_ESC = 124;
+		public const int DIGIT_GROUP = 125;
+		public const int REVERSE_DIGIT_GROUP = 126;
+		public const int ID_PREFIX = 127;
+		public const int ID_LETTER = 128;
+		public const int DIGIT = 129;
+		public const int HEXDIGIT = 130;
 		
 		
 	
@@ -227,71 +223,65 @@ namespace Boo.Lang.Parser
 			caseSensitiveLiterals = true;
 			setCaseSensitive(true);
 			literals = new Hashtable(100, (float) 0.4, null, Comparer.Default);
-			literals.Add("public", 49);
-			literals.Add("namespace", 48);
-			literals.Add("break", 11);
-			literals.Add("while", 70);
-			literals.Add("end", 23);
-			literals.Add("otherwise", 46);
-			literals.Add("raise", 52);
-			literals.Add("typeof", 65);
+			literals.Add("public", 46);
+			literals.Add("namespace", 45);
+			literals.Add("break", 10);
+			literals.Add("while", 64);
+			literals.Add("end", 22);
+			literals.Add("raise", 49);
+			literals.Add("typeof", 60);
 			literals.Add("and", 8);
-			literals.Add("failure", 28);
-			literals.Add("not", 43);
-			literals.Add("return", 54);
-			literals.Add("from", 30);
-			literals.Add("null", 44);
-			literals.Add("def", 18);
-			literals.Add("given", 34);
-			literals.Add("protected", 50);
-			literals.Add("ref", 53);
-			literals.Add("retry", 55);
-			literals.Add("when", 69);
-			literals.Add("class", 16);
-			literals.Add("do", 20);
-			literals.Add("except", 27);
-			literals.Add("event", 26);
-			literals.Add("ast", 10);
-			literals.Add("unless", 66);
-			literals.Add("super", 58);
-			literals.Add("set", 56);
-			literals.Add("transient", 63);
-			literals.Add("constructor", 17);
-			literals.Add("interface", 37);
-			literals.Add("is", 39);
-			literals.Add("internal", 38);
-			literals.Add("final", 29);
-			literals.Add("yield", 71);
-			literals.Add("or", 45);
-			literals.Add("destructor", 19);
-			literals.Add("if", 41);
-			literals.Add("success", 60);
-			literals.Add("override", 47);
+			literals.Add("not", 40);
+			literals.Add("return", 51);
+			literals.Add("from", 28);
+			literals.Add("null", 41);
+			literals.Add("def", 17);
+			literals.Add("protected", 47);
+			literals.Add("ref", 50);
+			literals.Add("class", 15);
+			literals.Add("do", 19);
+			literals.Add("except", 26);
+			literals.Add("event", 25);
+			literals.Add("unless", 61);
+			literals.Add("super", 54);
+			literals.Add("set", 52);
+			literals.Add("transient", 58);
+			literals.Add("constructor", 16);
+			literals.Add("interface", 34);
+			literals.Add("of", 42);
+			literals.Add("is", 36);
+			literals.Add("internal", 35);
+			literals.Add("final", 27);
+			literals.Add("yield", 65);
+			literals.Add("or", 43);
+			literals.Add("destructor", 18);
+			literals.Add("if", 38);
+			literals.Add("override", 44);
 			literals.Add("as", 9);
-			literals.Add("try", 62);
-			literals.Add("goto", 35);
-			literals.Add("enum", 25);
-			literals.Add("isa", 40);
-			literals.Add("for", 31);
-			literals.Add("char", 15);
-			literals.Add("private", 51);
-			literals.Add("false", 32);
-			literals.Add("static", 59);
+			literals.Add("try", 57);
+			literals.Add("goto", 32);
+			literals.Add("enum", 24);
+			literals.Add("isa", 37);
+			literals.Add("for", 29);
+			literals.Add("char", 14);
+			literals.Add("private", 48);
+			literals.Add("false", 30);
+			literals.Add("static", 55);
 			literals.Add("abstract", 7);
-			literals.Add("partial", 68);
-			literals.Add("callable", 13);
-			literals.Add("get", 33);
-			literals.Add("continue", 12);
-			literals.Add("cast", 14);
-			literals.Add("struct", 61);
-			literals.Add("else", 22);
-			literals.Add("import", 36);
-			literals.Add("elif", 21);
-			literals.Add("in", 42);
-			literals.Add("self", 57);
-			literals.Add("ensure", 24);
-			literals.Add("true", 64);
-			literals.Add("virtual", 67);
+			literals.Add("partial", 63);
+			literals.Add("callable", 12);
+			literals.Add("get", 31);
+			literals.Add("continue", 11);
+			literals.Add("cast", 13);
+			literals.Add("struct", 56);
+			literals.Add("else", 21);
+			literals.Add("import", 33);
+			literals.Add("elif", 20);
+			literals.Add("in", 39);
+			literals.Add("self", 53);
+			literals.Add("ensure", 23);
+			literals.Add("true", 59);
+			literals.Add("virtual", 62);
 		}
 		
 		override public IToken nextToken()			//throws TokenStreamException
@@ -513,7 +503,7 @@ tryAgain:
 		_ttype = ID;
 		
 		{
-			if ((cached_LA1=='$'||cached_LA1=='?'||cached_LA1=='@'))
+			if ((cached_LA1=='?'||cached_LA1=='@'))
 			{
 				mID_PREFIX(false);
 			}
@@ -560,11 +550,6 @@ _loop4_breakloop:			;
 		
 		switch ( cached_LA1 )
 		{
-		case '$':
-		{
-			match('$');
-			break;
-		}
 		case '@':
 		{
 			match('@');
@@ -2521,7 +2506,7 @@ _loop111_breakloop:					;
 	private static long[] mk_tokenSet_0_()
 	{
 		long[] data = new long[3072];
-		data[0]=-9223371968135299072L;
+		data[0]=-9223372036854775808L;
 		data[1]=576460745995190271L;
 		for (int i = 2; i<=1022; i++) { data[i]=-1L; }
 		data[1023]=9223372036854775807L;
