@@ -30,6 +30,23 @@ namespace BooCompiler.Tests
 {
 	using System;
 	
+	public class ObsoleteClass
+	{
+		[Obsolete("It is." )]
+		public static int Bar = 42;
+		
+		[Obsolete("Indeed it is." )]
+		public static void Foo()
+		{
+		}
+		
+		[Obsolete("We said so.")]
+		public static int Baz
+		{
+			get { return 42; }
+		}
+	}
+	
 	public class ReturnDucks
 	{
 		public class DuckBase {}
@@ -110,8 +127,22 @@ namespace BooCompiler.Tests
 		}
 	}
 	
+	public struct ValueTypeOverrideBoolOperator
+	{
+		public static implicit operator bool(ValueTypeOverrideBoolOperator instance)
+		{
+			Console.WriteLine("ValueTypeOverrideBoolOperator.operator bool");
+			return false;
+		}
+	}
+	
 	public class ExtendsOverridenBoolOperator : OverrideBoolOperator
 	{
+		[Boo.Lang.DuckTypedAttribute]
+		public ExtendsOverridenBoolOperator GetFoo()
+		{
+			return new ExtendsOverridenBoolOperator();
+		}
 	}
 	
 	public class OverrideEqualityOperators
@@ -346,6 +377,21 @@ namespace BooCompiler.Tests
 		{
 			Console.WriteLine("BaseClass.Method1");
 		}
+		
+		//for BOO-632 regression test
+		protected int _protectedfield = 0;
+		protected int ProtectedProperty
+		{
+			get
+			{
+				return _protectedfield;
+			}
+			
+			set
+			{
+				_protectedfield = value;
+			}
+		}
 	}
 	
 	public class DerivedClass : BaseClass
